# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/TWC

Description:
This script collects Gray water footprint data from the UNESCO-IHE FTP server. The data has a
weekly temporal resolution and a spatial resolution of 0.08333333 degree. The
resulting tiff files are in the WGS84 projection.

Requirement:
The WA_FTP username and password must be filled in the WebAccounts.py.
Contact the Water Accounting Team to get access to our WA FTP server.

Developers:
This dataset is developed by the Twente Water Centre:
Mekonnen, Mesfin M., and Arjen Y. Hoekstra.
"Global gray water footprint and water pollution levels related to anthropogenic nitrogen loads to fresh water."
Environmental science & technology 49.21 (2015): 12860-12868.

Example:
from watertools.Collect import TWC
TWC.Gray_Water_Footprint(Dir='C:/Temp/', latlim=[50,54], lonlim=[3,7])

"""
# General modules
import numpy as np
import os
from ftplib import FTP

# Water Accounting Modules
import watertools
import watertools.General.raster_conversions as RC
import watertools.General.data_conversions as DC


def DownloadData(Dir, latlim, lonlim):
    """
    This scripts downloads Gray Water Footprint data from the UNESCO-IHE ftp server.
    This dataset is created by the Twente Water Centre.
    The output files display the total ET in mm for a period of one week.
    The name of the file corresponds to the first day of the week.

    Keyword arguments:
	 Dir -- 'C:/file/to/path/'
    lonlim -- [ymin, ymax] (values must be between -60 and 84)
    latlim -- [xmin, xmax] (values must be between -180 and 180)
    """
    # Check the latitude and longitude and otherwise set lat or lon on greatest extent
    if latlim[0] < -60 or latlim[1] > 84:
        print('Latitude above 84N or below 60S is not possible. Value set to maximum')
        latlim[0] = np.max(latlim[0],-60)
        latlim[1] = np.min(latlim[1],84)
    if lonlim[0] < -180 or lonlim[1] > 180:
        print('Longitude must be between 180E and 180W. Now value is set to maximum')
        lonlim[0] = np.max(lonlim[0],-180)
        lonlim[1] = np.min(lonlim[1],180)

    # Define directory and create it if not exists
    output_folder = os.path.join(Dir, 'TWC', 'GWF')
    if not os.path.exists(output_folder):
        os.makedirs(output_folder)

    # Define end filename
    filename_Out = "Gray_Water_Footprint_Fraction.tif"

    # Temporary filename for the downloaded global file
    End_filename = os.path.join(output_folder, filename_Out)

    # Download the data from FTP server if the file not exists
    if not os.path.exists(End_filename):
        Download_GWF_from_WA_FTP(output_folder, End_filename, lonlim, latlim)

def Download_GWF_from_WA_FTP(output_folder, filename_Out, lonlim, latlim):
    """
    This function retrieves GWF data for a given date from the
    ftp.wateraccounting.unesco-ihe.org server.

    Keyword arguments:
    output_folder -- name of the end file with the weekly ALEXI data
    End_filename -- name of the end file
    lonlim -- [ymin, ymax] (values must be between -60 and 70)
    latlim -- [xmin, xmax] (values must be between -180 and 180)
    """

    try:
        # Collect account and FTP information
        username, password = watertools.Functions.Random.Get_Username_PWD.GET("FTP_WA")
        ftpserver = "ftp.wateraccounting.unesco-ihe.org"

        # Set the file names and directories
        filename = "Gray_Water_Footprint.tif"
        local_filename = os.path.join(output_folder, filename)

        # Download data from FTP
        ftp=FTP(ftpserver)
        ftp.login(username,password)
        directory="/WaterAccounting_Guest/Static_WA_Datasets/"
        ftp.cwd(directory)
        lf = open(local_filename, "wb")
        ftp.retrbinary("RETR " + filename, lf.write)
        lf.close()

        # Clip extend out of world data
        dataset, Geo_out = RC.clip_data(local_filename, latlim, lonlim)

        # make geotiff file
        DC.Save_as_tiff(name = filename_Out, data = dataset, geo = Geo_out, projection = "WGS84")

        # delete old tif file
        os.remove(local_filename)

    except:
        print("file not exists")

    return



# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: watertools/Functions/Area_Conversions

"""

from watertools.Functions.Visualization import Compare_Rasters_One_Point, Compare_Rasters_One_Polygon, Picture, Histogram, Values

__all__ = ['Compare_Rasters_One_Point', 'Compare_Rasters_One_Polygon', 'Picture', 'Histogram', 'Values']

__version__ = '0.1'



# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/DEM
"""

# General modules
import os
from ftplib import FTP
import urllib
import numpy as np
from osgeo import gdal
import time

import watertools.General.data_conversions as DC
import watertools.General.raster_conversions as RC

def DownloadData(output_folder, latlim, lonlim, dataset, level = None):
    """
    This function downloads SoilGrids data from SoilGrids.org

    Keyword arguments:
    output_folder -- directory of the result
	 latlim -- [ymin, ymax] (values must be between -50 and 50)
    lonlim -- [xmin, xmax] (values must be between -180 and 180)
    level -- "sl1" .... "sl7"
    dataset -- ground dataset
    """
    
    dict_levels = dict()
    dict_levels["sl1"] = "0-5"
    dict_levels["sl2"] = "5-15"
    dict_levels["sl3"] = "15-30"
    dict_levels["sl4"] = "30-60"
    dict_levels["sl5"] = "60-100"
    dict_levels["sl6"] = "100-200"    
    
    if "conversion" in locals():
        del conversion
    
    # Define parameter depedent variables
    if dataset == "BULKDENSITY":
        nameEnd = os.path.join(output_folder, 'BulkDensity_%s_SoilGrids_kg-m-3.tif' %level)
        parameter = "bdod"
        conversion = 10 # cg/cm3 to kg/m3
        level_str = dict_levels[level]               
    if dataset == "NITROGEN":
        nameEnd = os.path.join(output_folder, 'Nitrogen_%s_SoilGrids_g_kg-1.tif' %level)        
        parameter = "nitrogen"   
        level_str = dict_levels[level]     
        conversion = 0.01         #cg/kg to g/kg        
    if dataset == "SOC":
        nameEnd = os.path.join(output_folder, 'SoilOrganicCarbonContent_%s_SoilGrids_g_kg.tif' %level)          
        parameter = "soc"
        level_str = dict_levels[level]      
        conversion = 0.1         #dg/kg to g/kg
    if dataset == "SOD":
        nameEnd = os.path.join(output_folder, 'SoilOrganicCarbonDensity_%s_SoilGrids_g_kg.tif' %level)        
        parameter = "ocd"
        conversion = 0.1          #dg/kg to g/kg
        level_str = dict_levels[level]             
    if dataset == "PH":
        nameEnd = os.path.join(output_folder, 'SoilPH_%s_SoilGrids_pH10.tif' %level) 
        parameter = "phh2o"       
        level_str = dict_levels[level]               
    if dataset == "CLAY":
        nameEnd = os.path.join(output_folder, 'ClayContentMassFraction_%s_SoilGrids_percentage.tif' %level)  
        parameter = "clay"   
        level_str = dict_levels[level]     
        conversion = 0.1          #g/kg to percentage         
    if dataset == "SAND":
        nameEnd = os.path.join(output_folder, 'SandContentMassFraction_%s_SoilGrids_percentage.tif' %level)        
        parameter = "sand"          
        level_str = dict_levels[level]    
        conversion = 0.1          #g/kg to percentage              
    if dataset == "SILT":
        nameEnd = os.path.join(output_folder, 'SiltContentMassFraction_%s_SoilGrids_percentage.tif' %level)     
        parameter = "silt"     
        level_str = dict_levels[level]    
        conversion = 0.1          #g/kg to percentage              

    if not os.path.exists(nameEnd):
            
        # Download, extract, and converts all the files to tiff files
        try:
                    
            url = "https://maps.isric.org/mapserv?map=/map/%s.map&SERVICE=WCS&VERSION=2.0.1&REQUEST=GetCoverage&COVERAGEID=%s_%scm_mean&FORMAT=image/tiff&SUBSET=long(%f,%f)&SUBSET=lat(%f,%f)&SUBSETTINGCRS=http://www.opengis.net/def/crs/EPSG/0/4326&OUTPUTCRS=http://www.opengis.net/def/crs/EPSG/0/4326"  %(parameter, parameter, level_str, lonlim[0], lonlim[1], latlim[0], latlim[1])      
            #url = "http://85.214.241.121:8080/geoserver/ows?service=WCS&version=2.0.1&request=GetCoverage&CoverageId=%s_M_%s250m&subset=Long(%f,%f)&subset=Lat(%f,%f)" %(dataset, level_name, lonlim[0], lonlim[1], latlim[0], latlim[1])
            print(url)
            urllib.request.urlretrieve(url, filename=nameEnd)
            
            if "conversion" in locals():
                print("Conversion is applied of %s" %conversion)
                dest = gdal.Open(nameEnd)
                geo = dest.GetGeoTransform()
                proj = "WGS84"
                Array = dest.GetRasterBand(1).ReadAsArray()
                del dest
                time.sleep(1)
                Array = np.float_(Array) * conversion
                
                try:
                    Array = RC.gap_filling(Array, 0)
                except:
                    pass
                
                DC.Save_as_tiff(nameEnd, Array, geo, proj)

        except:
            print("Was not able to create the wanted dataset")
        
    return()


    




# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/CHIRPS
"""

# import general python modules
import os
import requests
import numpy as np
import pandas as pd
from ftplib import FTP
from joblib import Parallel, delayed

# WA+ modules
import watertools.General.raster_conversions as RC
import watertools.General.data_conversions as DC

def DownloadData(Dir, Startdate, Enddate, latlim, lonlim, Waitbar, cores, TimeCase):
    """
    This function downloads CHIRPS daily or monthly data

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    latlim -- [ymin, ymax] (values must be between -50 and 50)
    lonlim -- [xmin, xmax] (values must be between -180 and 180)
    Waitbar -- 1 (Default) will print a waitbar
    cores -- The number of cores used to run the routine. It can be 'False'
             to avoid using parallel computing routines.
    TimeCase -- String equal to 'daily' or 'monthly'
    """
    # Define timestep for the timedates
    if TimeCase == 'daily':
        TimeFreq = 'D'
        output_folder = os.path.join(Dir, 'Precipitation', 'CHIRPS', 'Daily')
    elif TimeCase == 'monthly':
        TimeFreq = 'MS'
        output_folder = os.path.join(Dir, 'Precipitation', 'CHIRPS', 'Monthly')
    else:
        raise KeyError("The input time interval is not supported")

    # make directory if it not exists
    if not os.path.exists(output_folder):
        os.makedirs(output_folder)

	# check time variables
    if not Startdate:
        Startdate = pd.Timestamp('1981-01-01')
    if not Enddate:
        Enddate = pd.Timestamp('Now')

    # Create days
    Dates = pd.date_range(Startdate, Enddate, freq=TimeFreq)

    # Create Waitbar
    if Waitbar == 1:
        import watertools.Functions.Random.WaitbarConsole as WaitbarConsole
        total_amount = len(Dates)
        amount = 0
        WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)

    # Check space variables
    if latlim[0] < -50 or latlim[1] > 50:
        print('Latitude above 50N or below 50S is not possible.'
               ' Value set to maximum')
        latlim[0] = np.max(latlim[0], -50)
        latlim[1] = np.min(lonlim[1], 50)
    if lonlim[0] < -180 or lonlim[1] > 180:
        print('Longitude must be between 180E and 180W.'
               ' Now value is set to maximum')
        lonlim[0] = np.max(latlim[0], -180)
        lonlim[1] = np.min(lonlim[1], 180)

    # Define IDs
    yID = 2000 - np.int16(np.array([np.ceil((latlim[1] + 50)*20),
                                    np.floor((latlim[0] + 50)*20)]))
    xID = np.int16(np.array([np.floor((lonlim[0] + 180)*20),
                             np.ceil((lonlim[1] + 180)*20)]))

    # Pass variables to parallel function and run
    args = [output_folder, TimeCase, xID, yID, lonlim, latlim]
    if not cores:
        for Date in Dates:
            RetrieveData(Date, args)
            if Waitbar == 1:
                amount += 1
                WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)
        results = True
    else:
        results = Parallel(n_jobs=cores)(delayed(RetrieveData)(Date, args)
                                         for Date in Dates)
    return results


def RetrieveData(Date, args):
    """
    This function retrieves CHIRPS data for a given date from the
    ftp://chg-ftpout.geog.ucsb.edu server.

    Keyword arguments:
    Date -- 'yyyy-mm-dd'
    args -- A list of parameters defined in the DownloadData function.
    """
    # Argument
    [output_folder, TimeCase, xID, yID, lonlim, latlim] = args

	# create all the input name (filename) and output (outfilename, filetif, DiFileEnd) names
    if TimeCase == 'daily':
        filename = 'chirps-v2.0.%s.%02s.%02s.tif.gz' %(Date.strftime('%Y'), Date.strftime('%m'), Date.strftime('%d'))
        filename2 = 'chirps-v2.0.%s.%02s.%02s.tif' %(Date.strftime('%Y'), Date.strftime('%m'), Date.strftime('%d'))        
        outfilename = os.path.join(output_folder,'chirps-v2.0.%s.%02s.%02s.tif' %(Date.strftime('%Y'), Date.strftime('%m'), Date.strftime('%d')))
        DirFileEnd = os.path.join(output_folder,'P_CHIRPS.v2.0_mm-day-1_daily_%s.%02s.%02s.tif' %(Date.strftime('%Y'), Date.strftime('%m'), Date.strftime('%d')))
    elif TimeCase == 'monthly':
        filename = 'chirps-v2.0.%s.%02s.tif.gz' %(Date.strftime('%Y'), Date.strftime('%m'))
        filename2 = 'chirps-v2.0.%s.%02s.tif' %(Date.strftime('%Y'), Date.strftime('%m'))
        outfilename = os.path.join(output_folder,'chirps-v2.0.%s.%02s.tif' %(Date.strftime('%Y'), Date.strftime('%m')))
        DirFileEnd = os.path.join(output_folder,'P_CHIRPS.v2.0_mm-month-1_monthly_%s.%02s.%02s.tif' %(Date.strftime('%Y'), Date.strftime('%m'), Date.strftime('%d')))
    else:
        raise KeyError("The input time interval is not supported")

    if not os.path.exists(DirFileEnd):

        try:
            # open ftp server
            ftp = FTP("chg-ftpout.geog.ucsb.edu", "", "")
            ftp.login()
        
        	# Define FTP path to directory
            if TimeCase == 'daily':
                pathFTP = 'pub/org/chg/products/CHIRPS-2.0/global_daily/tifs/p05/%s/' %Date.strftime('%Y')
            elif TimeCase == 'monthly':
                pathFTP = 'pub/org/chg/products/CHIRPS-2.0/global_monthly/tifs/'
            else:
                raise KeyError("The input time interval is not supported")
        
            # find the document name in this directory
            ftp.cwd(pathFTP)
            listing = []
        
        	# read all the file names in the directory
            ftp.retrlines("LIST", listing.append)
    
            # download the global rainfall file
            local_filename = os.path.join(output_folder, filename)
            lf = open(local_filename, "wb")
            ftp.retrbinary("RETR " + filename, lf.write, 8192)
            lf.close()
            
        except:
            if TimeCase == 'daily':
                url = os.path.join("https://data.chc.ucsb.edu/products/CHIRPS-2.0/global_daily/tifs/p05/%s/" %Date.strftime('%Y'), filename)
            elif TimeCase == 'monthly':
                url = os.path.join('https://data.chc.ucsb.edu/products/CHIRPS-2.0/global_monthly/tifs/', filename) 
            else:
                raise KeyError("The input time interval is not supported")
            
            try:
                local_filename = os.path.join(output_folder, filename)
                session = requests.Session()
                with session.get(url, stream=True) as r:
                    r.raise_for_status()
                    with open(local_filename, 'wb') as f:
                        for chunk in r.iter_content(chunk_size=1024 * 1024):
                            if chunk:  # filter out keep-alive new chunks
                                f.write(chunk)
               
                no_extract = 0                
            except:
                
                if TimeCase == 'daily':
                    url = os.path.join("https://data.chc.ucsb.edu/products/CHIRPS-2.0/global_daily/tifs/p05/%s/" %Date.strftime('%Y'), filename2)
                elif TimeCase == 'monthly':
                    url = os.path.join('https://data.chc.ucsb.edu/products/CHIRPS-2.0/global_monthly/tifs/', filename2) 
                else:
                    raise KeyError("The input time interval is not supported")    
                
                local_filename = os.path.join(output_folder, filename2)
                session = requests.Session()
                with session.get(url, stream=True) as r:
                    r.raise_for_status()
                    with open(local_filename, 'wb') as f:
                        for chunk in r.iter_content(chunk_size=1024 * 1024):
                            if chunk:  # filter out keep-alive new chunks
                                f.write(chunk)
                no_extract = 1
            
        try:
    
            if no_extract == 0:  
                # unzip the file
                zip_filename = os.path.join(output_folder, filename)
                DC.Extract_Data_gz(zip_filename, outfilename)
     
            # open tiff file
            dataset = RC.Open_tiff_array(outfilename)
    
            # clip dataset to the given extent
            data = dataset[yID[0]:yID[1], xID[0]:xID[1]]
            data[data < 0] = -9999
    
            # save dataset as geotiff file
            geo = [lonlim[0], 0.05, 0, latlim[1], 0, -0.05]
            DC.Save_as_tiff(name=DirFileEnd, data=data, geo=geo, projection="WGS84")
    
            # delete old tif file
            os.remove(outfilename)
    
        except:
            print("file not exists")
            
    return True



"""
Module: Collect/CFSR
"""
# General modules
import os
import pycurl
import datetime

def Download_data(Date, Version, output_folder, Var):
    """
    This function downloads CFSR data from the FTP server
				For - CFSR:    https://nomads.ncdc.noaa.gov/data/cfsr/
				    - CFSRv2:  http://nomads.ncdc.noaa.gov/modeldata/cfsv2_analysis_timeseries/

    Keyword arguments:
    Date -- pandas timestamp day
    Version -- 1 or 2 (1 = CFSR, 2 = CFSRv2)
    output_folder -- The directory for storing the downloaded files
    Var -- The variable that must be downloaded from the server ('dlwsfc','uswsfc','dswsfc','ulwsfc')
    """
    # Define the filename that must be downloaded
    if Version == 1:
        filename = Var + '.gdas.' + str(Date.strftime('%Y')) + str(Date.strftime('%m')) + '.grb2'
    if Version == 2:
        #if Date < datetime.datetime(2018, 8, 1):
        filename = Var + '.gdas.' + str(Date.strftime('%Y')) + str(Date.strftime('%m')) + '.grib2'
        #else:
        #    filename = "%s%02d"%(Date.year, Date.month) + Var + '.gdas.' + str(Date.strftime('%Y')) + str(Date.strftime('%m')) + '.grib2'
            
    try:
         # download the file when it not exist
        local_filename = os.path.join(output_folder, filename)
        #print(local_filename)
        if not os.path.exists(local_filename):
            Downloaded = 0
            Times = 0
            while Downloaded == 0:
                # Create the command and run the command in cmd
                if Version == 1:
                    #print("VERSION1")
                    #FTP_name = 'https://nomads.ncdc.noaa.gov/data/cfsr/' + Date.strftime('%Y') + Date.strftime('%m')+ '/' + filename
                    FTP_name = 'https://www.ncei.noaa.gov/data/climate-forecast-system/access/reanalysis/time-series/' + Date.strftime('%Y') + Date.strftime('%m')+ '/' + filename
                    #print(FTP_name)   
                if Version == 2:
                    #FTP_name = 'https://nomads.ncdc.noaa.gov/modeldata/cfsv2_analysis_timeseries/' + Date.strftime('%Y') + '/' + Date.strftime('%Y') + Date.strftime('%m')+ '/' + filename
                    FTP_name = 'https://www.ncei.noaa.gov/data/climate-forecast-system/access/operational-analysis/time-series/' + Date.strftime('%Y') + '/' + Date.strftime('%Y') + Date.strftime('%m')+ '/' + filename
                    #print(FTP_name)   
                     
                curl = pycurl.Curl()
                curl.setopt(pycurl.URL, FTP_name)
                fp = open(local_filename, "wb")
                curl.setopt(pycurl.SSL_VERIFYPEER, 0)
                curl.setopt(pycurl.SSL_VERIFYHOST, 0)
                curl.setopt(pycurl.WRITEDATA, fp)
                curl.perform()
                curl.close()
                fp.close()
                statinfo = os.stat(local_filename)
                if int(statinfo.st_size) > 10000:
                    Downloaded = 1
                else:
                    Times += 1
                    if Times == 10:
                        Downloaded = 1
    except:
        print('Was not able to download the CFSR file from the FTP server')

    return(local_filename)


# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Contact: timhessels@hotmail.com
Repository: https://github.com/TimHessels/watertools
Module: Collect/L30

Description:
This module downloads the Harmonized L30 data.
"""

from .Harmonized import main as Harmonized

__all__ = ['Harmonized']

__version__ = '0.1'



# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/FEWS

Description:
Use the FEWS.ETpot_daily function to
download and create monthly FEWS images in Gtiff format.
Data is available between 2003-01-01 till now. 

Examples:
from watertools.Collect import FEWS
FEWS.ETpot_daily(Dir='C:/Temp/', Startdate='2008-12-01', Enddate='2011-01-20',
           latlim=[-10, 30], lonlim=[-20, -10])
"""

from .ETpot_daily import main as ETpot_daily
__all__ = ['ETpot_daily']

__version__ = '0.1'



# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/TRMM


Description:
This module downloads TRMM3B42 V7 (daily) and TRMM3B43 V7 (monthly) data from
ftp://disc2.nascom.nasa.gov. Use the TRMM.daily or TRMM.monthly functions to
download and create daily or monthly TRMM images in Gtiff format.
The TRMM data is available since 1998-01-01 till 2015-04-30

Examples:
from watertools.Collect import TRMM
TRMM.daily(Dir='C:/Temp/', Startdate='1999-02-01', Enddate='1999-02-28',
           latlim=[-10, 30], lonlim=[-20, 120])
TRMM.monthly(Dir='C:/Temp/', Startdate='1999-02-01', Enddate='1999-02-28',
             latlim=[-10, 30], lonlim=[-20, 120])
"""

from watertools.Collect.RFE.daily import main as daily
from watertools.Collect.RFE.monthly import main as monthly

__all__ = ['daily', 'monthly']

__version__ = '0.1'



# -*- coding: utf-8 -*-
import sys
from watertools.Collect.NLDAS.FORA_DataAccess import DownloadData


def main(Dir, Vars, Startdate, Enddate, latlim, lonlim, cores=False,
         SumMean=1, Min=0, Max=0, Waitbar = 1):
    """
    This function downloads NLDAS daily data for a given variable, time
    interval, and spatial extent.

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    Vars -- ['tmp2m','spfh2m'] (array of strings) Variable code: VariablesInfo('day').descriptions.keys()
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    latlim -- [ymin, ymax]
    lonlim -- [xmin, xmax]
    SumMean -- 0 or 1. Indicates if the output values are the daily mean for
               instantaneous values or sum for fluxes
    Min -- 0 or 1. Indicates if the output values are the daily minimum
    Max -- 0 or 1. Indicates if the output values are the daily maximum
    Waitbar -- 1 (Default) Will print a waitbar

    """
    for Var in Vars:

        if Waitbar == 1:
            print('\nDownloading daily NLDAS %s data for the period %s till %s' %(Var, Startdate, Enddate))

        # Download data
        DownloadData(Dir, Var, Startdate, Enddate, latlim, lonlim, Waitbar, cores,
					 TimeCase='daily', CaseParameters=[SumMean, Min, Max])

if __name__ == '__main__':
    main(sys.argv)




# -*- coding: utf-8 -*-
"""
Created on Sun Dec 18 13:07:32 2016

@author: tih
"""
import gzip
import zipfile
import tarfile
from osgeo import gdal, osr
import os
import pandas as pd
import numpy as np
import netCDF4
import time

def Convert_nc_to_tiff(input_nc, output_folder):
    """
    This function converts the nc file into tiff files

    Keyword Arguments:
    input_nc -- name, name of the adf file
    output_folder -- Name of the output tiff file
    """
    from datetime import date
    import watertools.General.raster_conversions as RC

    #All_Data = RC.Open_nc_array(input_nc)

    if type(input_nc) == str:
        nc = netCDF4.Dataset(input_nc)
    elif type(input_nc) == list:
        nc = netCDF4.MFDataset(input_nc)

    Var = nc.variables.keys()[-1]
    All_Data = nc[Var]

    geo_out, epsg, size_X, size_Y, size_Z, Time = RC.Open_nc_info(input_nc)

    if epsg == 4326:
        epsg = 'WGS84'

    # Create output folder if needed
    if not os.path.exists(output_folder):
        os.mkdir(output_folder)

    for i in range(0,size_Z):
        if not Time == -9999:
            time_one = Time[i]
            d = date.fromordinal(time_one)
            name = os.path.splitext(os.path.basename(input_nc))[0]
            nameparts = name.split('_')[0:-2]
            name_out = os.path.join(output_folder, '_'.join(nameparts) + '_%d.%02d.%02d.tif' %(d.year, d.month, d.day))
            Data_one = All_Data[i,:,:]
        else:
            name=os.path.splitext(os.path.basename(input_nc))[0]
            name_out = os.path.join(output_folder, name + '.tif')
            Data_one = All_Data[:,:]

        Save_as_tiff(name_out, Data_one, geo_out, epsg)

    return()

def Convert_grb2_to_nc(input_wgrib, output_nc, band):
    '''
    import watertools.General.raster_conversions as RC
    
    # Get environmental variable
    WA_env_paths = os.environ["WA_PATHS"].split(';')
    GDAL_env_path = WA_env_paths[0]
    GDAL_TRANSLATE_PATH = os.path.join(GDAL_env_path, 'gdal_translate.exe')

    # Create command
    fullCmd = ' '.join(['"%s" -of netcdf -b %d' %(GDAL_TRANSLATE_PATH, band), input_wgrib, output_nc])  # -r {nearest}

    RC.Run_command_window(fullCmd)
    '''
    
    options_list = ['-of netcdf -b %d' %band]
    options_string = " ".join(options_list)
    gdal.UseExceptions()
    gdal.Translate(output_nc, input_wgrib, options = options_string)    

    return()

def Convert_adf_to_tiff(input_adf, output_tiff):
    """
    This function converts the adf files into tiff files

    Keyword Arguments:
    input_adf -- name, name of the adf file
    output_tiff -- Name of the output tiff file

    import watertools.General.raster_conversions as RC

    # Get environmental variable
    WA_env_paths = os.environ["WA_PATHS"].split(';')
    GDAL_env_path = WA_env_paths[0]
    GDAL_TRANSLATE_PATH = os.path.join(GDAL_env_path, 'gdal_translate.exe')

    # convert data from ESRI GRID to GeoTIFF
    fullCmd = ('"%s" -co COMPRESS=DEFLATE -co PREDICTOR=1 -co '
                   'ZLEVEL=1 -of GTiff %s %s') % (GDAL_TRANSLATE_PATH, input_adf, output_tiff)

    RC.Run_command_window(fullCmd)
    """
    
    options_list = ['-co COMPRESS=DEFLATE -co PREDICTOR=1 -co ZLEVEL=1']
    options_string = " ".join(options_list)
    gdal.UseExceptions()
    gdal.Translate(output_tiff, input_adf, options = options_string)

    return(output_tiff)
    
def Convert_bil_to_tiff(input_bil, output_tiff):
    """
    This function converts the bil files into tiff files

    Keyword Arguments:
    input_bil -- name, name of the bil file
    output_tiff -- Name of the output tiff file
    """
    import gdalconst
    
    gdal.GetDriverByName('EHdr').Register()
    dest = gdal.Open(input_bil, gdalconst.GA_ReadOnly)
    Array = dest.GetRasterBand(1).ReadAsArray()
    geo_out = dest.GetGeoTransform()
    Save_as_tiff(output_tiff, Array, geo_out, "WGS84")  
    
    return(output_tiff)
    
def Convert_hdf5_to_tiff(inputname_hdf, Filename_tiff_end, Band_number, scaling_factor, geo_out):
    """
    This function converts the hdf5 files into tiff files

    Keyword Arguments:
    input_adf -- name, name of the adf file
    output_tiff -- Name of the output tiff file
    Band_number -- bandnumber of the hdf5 that needs to be converted
    scaling_factor -- factor multipied by data is the output array
    geo -- [minimum lon, pixelsize, rotation, maximum lat, rotation,
            pixelsize], (geospatial dataset)
    """
    import watertools.General.raster_conversions as RC
    
    # Open the hdf file
    g = gdal.Open(inputname_hdf, gdal.GA_ReadOnly)

    #  Define temporary file out and band name in
    name_in = g.GetSubDatasets()[Band_number][0]

    # Get environmental variable
    WA_env_paths = os.environ["WA_PATHS"].split(';')
    GDAL_env_path = WA_env_paths[0]
    GDAL_TRANSLATE = os.path.join(GDAL_env_path, 'gdal_translate.exe')

    # run gdal translate command
    FullCmd = '%s -of GTiff %s %s' %(GDAL_TRANSLATE, name_in, Filename_tiff_end)
    RC.Run_command_window(FullCmd)

    # Get the data array
    dest = gdal.Open(Filename_tiff_end)
    Data = dest.GetRasterBand(1).ReadAsArray()
    dest = None
    
    # If the band data is not SM change the DN values into PROBA-V values and write into the spectral_reflectance_PROBAV
    Data_scaled = Data * scaling_factor

    # Save the PROBA-V as a tif file
    Save_as_tiff(Filename_tiff_end, Data_scaled, geo_out, "WGS84")    
    
    return()

def Extract_Data(input_file, output_folder):
    """
    This function extract the zip files

    Keyword Arguments:
    output_file -- name, name of the file that must be unzipped
    output_folder -- Dir, directory where the unzipped data must be
                           stored
    """
    # extract the data
    z = zipfile.ZipFile(input_file, 'r')
    z.extractall(output_folder)
    z.close()

def Extract_Data_gz(zip_filename, outfilename):
    """
    This function extract the zip files

    Keyword Arguments:
    zip_filename -- name, name of the file that must be unzipped
    outfilename -- Dir, directory where the unzipped data must be
                           stored
    """

    with gzip.GzipFile(zip_filename, 'rb') as zf:
        file_content = zf.read()
        save_file_content = open(outfilename, 'wb')
        save_file_content.write(file_content)
    save_file_content.close()
    zf.close()
    os.remove(zip_filename)
    
def Extract_Data_tar_gz(zip_filename, output_folder):
    """
    This function extract the tar.gz files

    Keyword Arguments:
    zip_filename -- name, name of the file that must be unzipped
    output_folder -- Dir, directory where the unzipped data must be
                           stored
    """

    os.chdir(output_folder)
    tar = tarfile.open(zip_filename, "r:gz")
    tar.extractall()
    tar.close()

def Extract_Data_tar(zip_filename, output_folder):
    """
    This function extract the tar files

    Keyword Arguments:
    zip_filename -- name, name of the file that must be unzipped
    output_folder -- Dir, directory where the unzipped data must be
                           stored
    """

    os.chdir(output_folder)
    tar = tarfile.open(zip_filename, "r")
    tar.extractall()
    tar.close()
    
def Save_as_tiff(name='', data='', geo='', projection=''):
    """
    This function save the array as a geotiff

    Keyword arguments:
    name -- string, directory name
    data -- [array], dataset of the geotiff
    geo -- [minimum lon, pixelsize, rotation, maximum lat, rotation,
            pixelsize], (geospatial dataset)
    projection -- integer, the EPSG code
    """
    
    dir_name = os.path.dirname(name)
    
    if not os.path.exists(dir_name):
        success = 0
        while success == 0:
            try:
                os.makedirs(dir_name)
                success = 1
            except:
                time.sleep(1)
            
    # save as a geotiff
    driver = gdal.GetDriverByName("GTiff")
    dst_ds = driver.Create(name, int(data.shape[1]), int(data.shape[0]), 1, gdal.GDT_Float32, ['COMPRESS=LZW'])
    srse = osr.SpatialReference()
    if projection == '':
        srse.SetWellKnownGeogCS("WGS84")

    else:
        try:
            if not srse.SetWellKnownGeogCS(projection) == 6:
                srse.SetWellKnownGeogCS(projection)
            else:
                try:
                    srse.ImportFromEPSG(int(projection))
                except:
                    srse.ImportFromWkt(projection)
        except:
            try:
                srse.ImportFromEPSG(int(projection))
            except:
                srse.ImportFromWkt(projection)

    dst_ds.SetProjection(srse.ExportToWkt())
    dst_ds.GetRasterBand(1).SetNoDataValue(-9999)
    dst_ds.SetGeoTransform(geo)
    dst_ds.GetRasterBand(1).WriteArray(data)
    dst_ds = None
    return()

def Save_as_MEM(data='', geo='', projection=''):
    """
    This function save the array as a memory file

    Keyword arguments:
    data -- [array], dataset of the geotiff
    geo -- [minimum lon, pixelsize, rotation, maximum lat, rotation,
            pixelsize], (geospatial dataset)
    projection -- interger, the EPSG code
    """
    # save as a geotiff
    driver = gdal.GetDriverByName("MEM")
    dst_ds = driver.Create('', int(data.shape[1]), int(data.shape[0]), 1,
                           gdal.GDT_Float32)
    srse = osr.SpatialReference()
    if projection == '':
        srse.SetWellKnownGeogCS("WGS84")

    else:
        try:
            if not srse.SetWellKnownGeogCS(projection) == 6:
                srse.SetWellKnownGeogCS(projection)
            else:
                try:
                    srse.ImportFromEPSG(int(projection))
                except:
                    srse.ImportFromWkt(projection)
        except:
            try:
                srse.ImportFromEPSG(int(projection))
            except:
                srse.ImportFromWkt(projection)
    dst_ds.SetProjection(srse.ExportToWkt())
    dst_ds.GetRasterBand(1).SetNoDataValue(-9999)
    dst_ds.SetGeoTransform(geo)
    dst_ds.GetRasterBand(1).WriteArray(data)
    return(dst_ds)

def Save_as_NC(namenc, DataCube, Var, Reference_filename,  Startdate = '', Enddate = '', Time_steps = '', Scaling_factor = 1):
    """
    This function save the array as a netcdf file

    Keyword arguments:
    namenc -- string, complete path of the output file with .nc extension
    DataCube -- [array], dataset of the nc file, can be a 2D or 3D array [time, lat, lon], must be same size as reference data
    Var -- string, the name of the variable
    Reference_filename -- string, complete path to the reference file name
    Startdate -- 'YYYY-mm-dd', needs to be filled when you want to save a 3D array,  defines the Start datum of the dataset
    Enddate -- 'YYYY-mm-dd', needs to be filled when you want to save a 3D array, defines the End datum of the dataset
    Time_steps -- 'monthly' or 'daily', needs to be filled when you want to save a 3D array, defines the timestep of the dataset
    Scaling_factor -- number, scaling_factor of the dataset, default = 1
    """
    # Import modules
    import watertools.General.raster_conversions as RC
    from netCDF4 import Dataset

    if not os.path.exists(namenc):

        # Get raster information
        geo_out, proj, size_X, size_Y = RC.Open_array_info(Reference_filename)

        # Create the lat/lon rasters
        lon = np.arange(size_X)*geo_out[1]+geo_out[0] - 0.5 * geo_out[1]
        lat = np.arange(size_Y)*geo_out[5]+geo_out[3] - 0.5 * geo_out[5]

        # Create the nc file
        nco = Dataset(namenc, 'w', format='NETCDF4_CLASSIC')
        nco.description = '%s data' %Var

        # Create dimensions, variables and attributes:
        nco.createDimension('longitude', size_X)
        nco.createDimension('latitude', size_Y)

        # Create time dimension if the parameter is time dependent
        if Startdate != '':
            if Time_steps == 'monthly':
                Dates = pd.date_range(Startdate,Enddate,freq = 'MS')
            if Time_steps == 'daily':
                Dates = pd.date_range(Startdate,Enddate,freq = 'D')
            time_or=np.zeros(len(Dates))
            i = 0
            for Date in Dates:
                time_or[i] = Date.toordinal()
                i += 1
            nco.createDimension('time', None)
            timeo = nco.createVariable('time', 'f4', ('time',))
            timeo.units = '%s' %Time_steps
            timeo.standard_name = 'time'

        # Create the lon variable
        lono = nco.createVariable('longitude', 'f8', ('longitude',))
        lono.standard_name = 'longitude'
        lono.units = 'degrees_east'
        lono.pixel_size = geo_out[1]

        # Create the lat variable
        lato = nco.createVariable('latitude', 'f8', ('latitude',))
        lato.standard_name = 'latitude'
        lato.units = 'degrees_north'
        lato.pixel_size = geo_out[5]

        # Create container variable for CRS: lon/lat WGS84 datum
        crso = nco.createVariable('crs', 'i4')
        crso.long_name = 'Lon/Lat Coords in WGS84'
        crso.grid_mapping_name = 'latitude_longitude'
        crso.projection = proj
        crso.longitude_of_prime_meridian = 0.0
        crso.semi_major_axis = 6378137.0
        crso.inverse_flattening = 298.257223563
        crso.geo_reference = geo_out

        # Create the data variable
        if Startdate != '':
            preco = nco.createVariable('%s' %Var, 'f8',  ('time', 'latitude', 'longitude'), zlib=True, least_significant_digit=1)
            timeo[:]=time_or
        else:
            preco = nco.createVariable('%s' %Var, 'f8',  ('latitude', 'longitude'), zlib=True, least_significant_digit=1)

        # Set the data variable information
        preco.scale_factor = Scaling_factor
        preco.add_offset = 0.00
        preco.grid_mapping = 'crs'
        preco.set_auto_maskandscale(False)

        # Set the lat/lon variable
        lono[:] = lon
        lato[:] = lat

        # Set the data variable
        if Startdate != '':
            for i in range(len(Dates)):
                preco[i,:,:] = DataCube[i,:,:]*1./np.float(Scaling_factor)
        else:
            preco[:,:] = DataCube[:,:] * 1./np.float(Scaling_factor)

        nco.close()
    return()

def Create_NC_name(Var, Simulation, Dir_Basin, sheet_nmbr, info = ''):

    # Create the output name
    nameOut=''.join(['_'.join([Var,'Simulation%d' % Simulation,'_'.join(info)]),'.nc'])
    namePath = os.path.join(Dir_Basin,'Simulations','Simulation_%d' %Simulation, 'Sheet_%d' %sheet_nmbr)
    if not os.path.exists(namePath):
        os.makedirs(namePath)
    nameTot=os.path.join(namePath,nameOut)

    return(nameTot)

def Create_new_NC_file(nc_outname, Basin_Example_File, Basin):

    # Open basin file
    dest = gdal.Open(Basin_Example_File)
    Basin_array = dest.GetRasterBand(1).ReadAsArray()
    Basin_array[np.isnan(Basin_array)] = -9999
    Basin_array[Basin_array<0] = -9999

    # Get Basic information
    Geo = dest.GetGeoTransform()
    size_X = dest.RasterXSize
    size_Y = dest.RasterYSize
    epsg = dest.GetProjection()

    # Get Year and months
    year = int(os.path.basename(nc_outname).split(".")[0])
    Dates = pd.date_range("%d-01-01" %year, "%d-12-31" %year, freq = "MS")

    # Latitude and longitude
    lons = np.arange(size_X)*Geo[1]+Geo[0] + 0.5 * Geo[1]
    lats = np.arange(size_Y)*Geo[5]+Geo[3] + 0.5 * Geo[5]

    # Create NetCDF file
    nco = netCDF4.Dataset(nc_outname, 'w', format = 'NETCDF4_CLASSIC')
    nco.set_fill_on()
    nco.description = '%s' %Basin

    # Create dimensions
    nco.createDimension('latitude', size_Y)
    nco.createDimension('longitude', size_X)
    nco.createDimension('time', None)

    # Create NetCDF variables
    crso = nco.createVariable('crs', 'i4')
    crso.long_name = 'Lon/Lat Coords in WGS84'
    crso.standard_name = 'crs'
    crso.grid_mapping_name = 'latitude_longitude'
    crso.projection = epsg
    crso.longitude_of_prime_meridian = 0.0
    crso.semi_major_axis = 6378137.0
    crso.inverse_flattening = 298.257223563
    crso.geo_reference = Geo

    ######################### Save Rasters in NetCDF ##############################

    lato = nco.createVariable('latitude', 'f8', ('latitude',))
    lato.units = 'degrees_north'
    lato.standard_name = 'latitude'
    lato.pixel_size = Geo[5]

    lono = nco.createVariable('longitude', 'f8', ('longitude',))
    lono.units = 'degrees_east'
    lono.standard_name = 'longitude'
    lono.pixel_size = Geo[1]

    timeo = nco.createVariable('time', 'f4', ('time',))
    timeo.units = 'Monthly'
    timeo.standard_name = 'time'

    # Variables
    basin_var = nco.createVariable('Landuse', 'i',
                                           ('latitude', 'longitude'),
                                           fill_value=-9999)
    basin_var.long_name = 'Landuse'
    basin_var.grid_mapping = 'crs'

    # Create time unit
    i = 0
    time_or=np.zeros(len(Dates))
    for Date in Dates:
        time_or[i] = Date.toordinal()
        i += 1

    # Load data
    lato[:] = lats
    lono[:] = lons
    timeo[:] = time_or
    basin_var[:,:] = Basin_array

    # close the file
    time.sleep(1)
    nco.close()
    return()

def Add_NC_Array_Variable(nc_outname, Array, name, unit, Scaling_factor = 1):

    # create input array
    Array[np.isnan(Array)] = -9999 * np.float(Scaling_factor)
    Array = np.int_(Array * 1./np.float(Scaling_factor))

    # Create NetCDF file
    nco = netCDF4.Dataset(nc_outname, 'r+', format = 'NETCDF4_CLASSIC')
    nco.set_fill_on()

    paro = nco.createVariable('%s' %name, 'i',
                                   ('time', 'latitude', 'longitude'),fill_value=-9999,
                                    zlib=True, least_significant_digit=0)

    paro.scale_factor = Scaling_factor
    paro.add_offset = 0.00
    paro.grid_mapping = 'crs'
    paro.long_name = name
    paro.units = unit
    paro.set_auto_maskandscale(False)

    # Set the data variable
    paro[:,:,:] = Array

    # close the file
    time.sleep(1)
    nco.close()

    return()

def Add_NC_Array_Static(nc_outname, Array, name, unit, Scaling_factor = 1):

    # create input array
    Array[np.isnan(Array)] = -9999 * np.float(Scaling_factor)
    Array = np.int_(Array * 1./np.float(Scaling_factor))

    # Create NetCDF file
    nco = netCDF4.Dataset(nc_outname, 'r+', format = 'NETCDF4_CLASSIC')
    nco.set_fill_on()

    paro = nco.createVariable('%s' %name, 'i',
                                   ('latitude', 'longitude'),fill_value=-9999,
                                    zlib=True, least_significant_digit=0)

    paro.scale_factor = Scaling_factor
    paro.add_offset = 0.00
    paro.grid_mapping = 'crs'
    paro.long_name = name
    paro.units = unit
    paro.set_auto_maskandscale(False)

    # Set the data variable
    paro[:,:] = Array

    # close the file
    time.sleep(1)
    nco.close()

    return()

def Convert_dict_to_array(River_dict, Array_dict, Reference_data):

    import numpy as np
    import os
    import watertools.General.raster_conversions as RC

     
    if str(type(Reference_data)) == "<class 'osgeo.gdal.Dataset'>":    
        geo_out = Reference_data.GetGeoTransform()
        proj = Reference_data.GetProjection()
        size_Y = Reference_data.RasterYSize
        size_X = Reference_data.RasterXSize      

    elif os.path.splitext(Reference_data)[-1] == '.nc':
        # Get raster information
        geo_out, proj, size_X, size_Y, size_Z, Time = RC.Open_nc_info(Reference_data)
 
    else:
        # Get raster information
        geo_out, proj, size_X, size_Y = RC.Open_array_info(Reference_data)

    # Create ID Matrix
    y,x = np.indices((size_Y, size_X))
    ID_Matrix = np.int32(np.ravel_multi_index(np.vstack((y.ravel(),x.ravel())),(size_Y,size_X),mode='clip').reshape(x.shape)) + 1

    # Get tiff array time dimension:
    time_dimension = int(np.shape(Array_dict[0])[0])

    # create an empty array
    DataCube = np.ones([time_dimension, size_Y, size_X]) * np.nan

    for river_part in River_dict.keys():
        #print("river part %s" %river_part)
        for river_pixel in range(1,len(River_dict[river_part])):
            river_pixel_ID = River_dict[river_part][river_pixel]
            if len(np.argwhere(ID_Matrix == river_pixel_ID))>0:
                #print(np.argwhere(ID_Matrix == river_pixel_ID))
                row, col = np.argwhere(ID_Matrix == river_pixel_ID)[0][:]
                #print(np.max(Array_dict[river_part][:,river_pixel]))
                DataCube[:,row,col] = Array_dict[river_part][:,river_pixel]

    return(DataCube)




# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: General

Description:
This module consists of the general functions that are used in the WA+ toolbox
"""

from watertools.General import data_conversions, raster_conversions

__all__ = ['data_conversions','raster_conversions']

__version__ = '0.1'


# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/MSGCPP
"""

import sys
from watertools.Collect.MSGCPP.DataAccess import DownloadData

def main(Dir, Startdate, Enddate, latlim, lonlim, Time = '', GMT_Offset = 0, Waitbar = 1):
    """
    This function downloads MOD11 daily data for the specified time
    interval, and spatial extent.

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    latlim -- [ymin, ymax]
    lonlim -- [xmin, xmax]
    Time -- "12:45" Define the time, default is '' all 15min will be downloaded
    Waitbar -- 1 (Default) will print a waitbar
    """
    print('\nDownload 15min short downward surface radiation from MSGCPP for period %s till %s' %(Startdate, Enddate))
    
    DownloadData(Dir, Startdate, Enddate, latlim, lonlim, Time, GMT_Offset, Waitbar, Type = "SDS")

if __name__ == '__main__':
    main(sys.argv)


# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Function/Start
"""
# General Python modules
import numpy as np
import os
import glob
import pandas as pd
from osgeo import gdal

def Nearest_Interpolate(Dir_in, Startdate, Enddate, format_in = None, format_out = None, Dir_out = None, AOI = None):
    """
    This functions calculates yearly tiff files based on the monthly tiff files. (will calculate the total sum)

    Parameters
    ----------
    Dir_in : str
        Path to the input data
    Startdate : str
        Contains the start date of the model 'yyyy-mm-dd'
    Enddate : str
        Contains the end date of the model 'yyyy-mm-dd'
    Dir_out : str
        Path to the output data, default is same as Dir_in

    """
    # import WA+ modules
    import watertools.General.data_conversions as DC
    import watertools.General.raster_conversions as RC

    # Change working directory
    os.chdir(Dir_in)

    # Define end and start date
    Dates = pd.date_range(Startdate, Enddate, freq='AS')

    # Find all monthly files
    if format_in == None:
        files = glob.glob('*monthly*.tif')
    else:
        files = glob.glob(format_in.replace(":02d","").format(yyyy= "*", mm = "*", dd = "*"))

    # Get array information and define projection
    geo_out, proj, size_X, size_Y = RC.Open_array_info(files[0])
    if int(proj.split('"')[-2]) == 4326:
        proj = "WGS84"

    # Get the No Data Value
    dest = gdal.Open(files[0])
    NDV = dest.GetRasterBand(1).GetNoDataValue()

    for date in Dates:
        Year = date.year
        
        if format_in == None:
            files_one_year = glob.glob('*monthly*%d*.tif' %Year)
        else:
            files_one_year = glob.glob(format_in.replace(":02d","").format(yyyy=Year, mm = "*", dd = "*"))

        # Create empty arrays
        Year_data = np.zeros([size_Y, size_X])

        if len(files_one_year) is not int(12):
            print("One month in year %s is missing!" %Year)

        for file_one_year in files_one_year:
            file_path = os.path.join(Dir_in, file_one_year)

            Month_data = RC.Open_tiff_array(file_path)
            Month_data[np.isnan(Month_data)] = 0.0
            Month_data[Month_data == -9999] = 0.0
            Year_data += Month_data

        if str(type(AOI)) == "<class 'numpy.ndarray'>":
            Year_data = Year_data * AOI

        # Define output directory
        if Dir_out == None:
            Dir_out = Dir_in
        if not os.path.exists(Dir_out):
            os.makedirs(Dir_out)

        # Define output name
        if format_out == None:
            output_name = os.path.join(Dir_out, file_one_year.replace('monthly', 'yearly').replace('month','year'))
            output_name = output_name[:-14] + '%d.01.01.tif' %(date.year)
        else:
            output_name = os.path.join(Dir_out, format_out.format(yyyy = date.year,  mm = 1, dd = 1))

        # Save tiff file
        DC.Save_as_tiff(output_name, Year_data, geo_out, proj)

    return













# -*- coding: utf-8 -*-
'''
Authors: Tim Hessels
Module: Products/ETref
'''
# import general python modules
from osgeo import gdal
import numpy as np

# import watertools modules
from watertools.Products.ETref.Interpolate_Meteo_ETref import process_GLDAS, lapse_rate, adjust_P, slope_correct
import watertools.General.raster_conversions as RC

def calc_ETref(Dir, tmin_str, tmax_str, humid_str, press_str, wind_str, down_short_str, down_long_str, up_long_str, DEMmap_str, DOY):
    """
    This function calculates the ETref by using all the input parameters (path)
    according to FAO standards
    see: http://www.fao.org/docrep/x0490e/x0490e08.htm#TopOfPage

    Keyword arguments:
    tmin_str -- 'C:/'  path to the minimal temperature tiff file [degrees Celcius], e.g. from GLDAS
    tmax_str -- 'C:/'  path to the maximal temperature tiff file [degrees Celcius], e.g. from GLDAS
    humid_str -- 'C:/'  path to the humidity tiff file [kg/kg], e.g. from GLDAS
    press_str -- 'C:/'  path to the air pressure tiff file [kPa], e.g. from GLDAS
    wind_str -- 'C:/'  path to the wind velocity tiff file [m/s], e.g. from GLDAS
    down_short_str -- 'C:/'  path to the downward shortwave radiation tiff file [W*m-2], e.g. from CFSR/LANDSAF
    down_long_str -- 'C:/'  path to the downward longwave radiation tiff file [W*m-2], e.g. from CFSR/LANDSAF
    up_long_str -- 'C:/'  path to the upward longwave radiation tiff file [W*m-2], e.g. from CFSR/LANDSAF
    DEMmap_str -- 'C:/'  path to the DEM tiff file [m] e.g. from HydroSHED
    DOY -- Day of the year
    """

    # Get some geo-data to save results
    GeoT, Projection, xsize, ysize = RC.Open_array_info(DEMmap_str)
    #NDV, xsize, ysize, GeoT, Projection, DataType = GetGeoInfo(DEMmap_str)
    raster_shape = [xsize, ysize]

    # Create array to store results
    ETref = np.zeros(raster_shape)

    # gap fill
    tmin_str_GF = RC.gap_filling(tmin_str,-9999)
    tmax_str_GF = RC.gap_filling(tmax_str,-9999)
    humid_str_GF = RC.gap_filling(humid_str,-9999)
    press_str_GF = RC.gap_filling(press_str,-9999)
    wind_str_GF = RC.gap_filling(wind_str,-9999)
    down_short_str_GF = RC.gap_filling(down_short_str,np.nan)
    down_long_str_GF = RC.gap_filling(down_long_str,np.nan)
    if up_long_str is not 'not':
        up_long_str_GF = RC.gap_filling(up_long_str,np.nan)
    else:
        up_long_str_GF = 'nan'


    #dictionary containing all tthe paths to the input-maps
    inputs = dict({'tmin':tmin_str_GF,'tmax':tmax_str_GF,'humid':humid_str_GF,'press':press_str_GF,'wind':wind_str_GF,'down_short':down_short_str_GF,'down_long':down_long_str_GF,'up_long':up_long_str_GF})


    #dictionary containing numpy arrays of al initial and intermediate variables
    input_array = dict({'tmin':None,'tmax':None,'humid':None,'press':None,'wind':None,'albedo':None,'down_short':None,'down_long':None,'up_short':None,'up_long':None,'net_radiation':None,'ea':None,'es':None,'delta':None})

    #APPLY LAPSE RATE CORRECTION ON TEMPERATURE
    tmin = lapse_rate(Dir, inputs['tmin'], DEMmap_str)
    tmax = lapse_rate(Dir, inputs['tmax'], DEMmap_str)

    #PROCESS PRESSURE MAPS
    press =adjust_P(Dir, inputs['press'], DEMmap_str)

    #PREPARE HUMIDITY MAPS
    dest = RC.reproject_dataset_example(inputs['humid'], DEMmap_str, method = 2)
    humid=dest.GetRasterBand(1).ReadAsArray()
    dest = None

    #CORRECT WIND MAPS
    dest = RC.reproject_dataset_example(inputs['wind'], DEMmap_str,method = 2)
    wind=dest.GetRasterBand(1).ReadAsArray()*0.75
    dest = None

    #PROCESS GLDAS DATA
    input_array['ea'], input_array['es'], input_array['delta'] = process_GLDAS(tmax,tmin,humid,press)

    ea=input_array['ea']
    es=input_array['es']
    delta=input_array['delta']

    if up_long_str == 'not':

        #CORRECT WIND MAPS
        dest = RC.reproject_dataset_example(down_short_str, DEMmap_str,method = 2)
        Short_Net_data=dest.GetRasterBand(1).ReadAsArray()*0.75
        dest = None

        dest = RC.reproject_dataset_example(down_long_str, DEMmap_str,method = 2)
        Short_Clear_data=dest.GetRasterBand(1).ReadAsArray()*0.75
        dest = None

        # Calculate Long wave Net radiation
        Rnl = 4.903e-9 * (((tmin + 273.16)**4+(tmax + 273.16)**4)/2)*(0.34 - 0.14 * np.sqrt(ea)) * (1.35 * Short_Net_data/Short_Clear_data -0.35)

        # Calulate Net Radiation and converted to MJ*d-1*m-2
        net_radiation = (Short_Net_data * 0.77 + Rnl)*86400/10**6


    else:
        #OPEN DOWNWARD SHORTWAVE RADIATION
        dest = RC.reproject_dataset_example(inputs['down_short'], DEMmap_str,method = 2)
        down_short=dest.GetRasterBand(1).ReadAsArray()
        dest = None
        down_short, tau, bias = slope_correct(down_short,press,ea,DEMmap_str,DOY)

        #OPEN OTHER RADS
        up_short = down_short*0.23

        dest =  RC.reproject_dataset_example(inputs['down_long'], DEMmap_str,method = 2)
        down_long=dest.GetRasterBand(1).ReadAsArray()
        dest = None

        dest =  RC.reproject_dataset_example(inputs['up_long'], DEMmap_str,method = 2)
        up_long=dest.GetRasterBand(1).ReadAsArray()
        dest = None

        #OPEN NET RADIATION AND CONVERT W*m-2 TO MJ*d-1*m-2
        net_radiation = ((down_short-up_short) + (down_long-up_long))*86400/10**6


    #CALCULATE ETref
    ETref = (0.408 * delta * net_radiation + 0.665*10**-3 *
        press * (900/((tmax+tmin)/2 + 273)) *
        wind * (es - ea)) / (delta + 0.665*10**-3 *
        press * (1 + 0.34 * wind))

    # Set limits ETref
    ETref[ETref<0]=0
    ETref[ETref>400]=np.nan

    #return a reference ET map (numpy array), a dictionary containing all intermediate information and a bias of the slope correction on down_short
    return ETref






















# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Contact: timhessels@hotmail.com
Repository: https://github.com/TimHessels/watertools
Module: Collect/CFSR

Description:
This module downloads CFSR (daily or monthly) radiation data from
ftp://nomads.ncdc.noaa.gov. Use the CFSR.daily, CFSRv2.daily, CFSR.monthly, or CFSRv2.monthly functions to
download and create daily or monthly CSFR/CSFRv2 radiation images in Gtiff format.
The CFSR data is available since 1979-01-01 till 2011-03-31
The CFSRv2 data is available since 2011-04-01 till now

Examples:
from watertools.Collect import CFSR
CFSR.daily(Dir='C:/Temp/', Vars = ['dlwsfc'], Startdate='2011-03-01', Enddate='2011-04-15',
           latlim=[-10, 30], lonlim=[-20, 120])
!CFSR.monthly(Dir='C:/Temp/', ['uswsfc','dswsfc'], Startdate='1999-02-01', Enddate='1999-02-28',
!             latlim=[-10, 30], lonlim=[-20, 120]) (Still in progress at the moment!!!)
"""

from .daily import main as daily
#from .monthly import main as monthly

__all__ = ['daily']

__version__ = '0.1'


# -*- coding: utf-8 -*-
"""
WaterSat
author: Tim Martijn Hessels
Created on Sun Sep 15 18:40:44 2019
"""

import requests, json, time
import pandas as pd
import os
import urllib
import datetime
from pyproj import Proj, transform

import watertools

def WAPOR(output_folder, Startdate, Enddate, latlim, lonlim, Parameter, Area = None, Version = "2"):
    
    time_steps = Parameter.split("_")[-1]
    level = Parameter.split("_")[0]
    
    # Find the time frequency of the parameter
    if time_steps == "A":
        freq = "AS"
    if time_steps == "M":
        freq = "MS"    
    if time_steps == "D":
        freq = "MS"    
    
    # Find LEVEL type
    if level == "L3":
        Parameter_first_part = "_".join(Parameter.split("_")[0:-1])
        Parameter = "_".join([Parameter_first_part, "{AREA}", Parameter.split("_")[-1]])
        Version = "1" # LEVEL 3 is only available in Version 1 dataset
    
    # Define dates
    Dates = pd.date_range(Startdate, Enddate, freq = freq)
    
    if time_steps == "D":
        for Date in Dates:
            if Date == Dates[0]:
                Dates_end = [pd.Timestamp(datetime.datetime(Date.year, Date.month, 1)), pd.Timestamp(datetime.datetime(Date.year, Date.month, 11)), pd.Timestamp(datetime.datetime(Date.year, Date.month, 21))]
    
            else:
                Dates_end.append(pd.Timestamp(datetime.datetime(Date.year, Date.month, 1)))
                Dates_end.append(pd.Timestamp(datetime.datetime(Date.year, Date.month, 11)))
                Dates_end.append(pd.Timestamp(datetime.datetime(Date.year, Date.month, 21)))
    
    else:
        Dates_end = Dates   
    
    auth_token, pwd = watertools.Functions.Random.Get_Username_PWD.GET("WAPOR")
    
    # Define server         
    url = 'https://io.apps.fao.org/gismgr/api/v1/query'
           
    # Login into WAPOR
    sign_in= 'https://io.apps.fao.org/gismgr/api/v1/iam/sign-in'
    resp_vp=requests.post(sign_in,headers={'X-GISMGR-API-KEY':auth_token})
    resp_vp = resp_vp.json()
    token = resp_vp['response']['accessToken']
    
    # Set header type
    header = {"Authorization": "Bearer " + token,
              "Content-type": "application/json;charset=UTF-8",
              "Accept": "application/json"
             } 
    
    # Get constant variables for payload       
    measure = VariablesInfo.measures[Parameter]
    dimension = VariablesInfo.dimensions[Parameter]
    version = VariablesInfo.versions[Version]

    output_folder_para =  os.path.join(output_folder, Parameter.format(AREA = Area)) 
    if not os.path.exists(output_folder_para):
        os.makedirs(output_folder_para)
    
    if Parameter.split("_")[0] == "L1":
        latlim[0] = latlim[0] - 0.1
        latlim[1] = latlim[1] + 0.1
        lonlim[0] = lonlim[0] - 0.1
        lonlim[1] = lonlim[1] + 0.1
      
    if level == "L3":     
        Projection = LEVEL3.Projection[Area]
        inProj = Proj(init='epsg:4326')
        outProj = Proj(init='epsg:%d' %Projection)
        Projection = Projection
    
        lonlim[0], latlim[1] = transform(inProj, outProj, lonlim[0], latlim[1])
        lonlim[1], latlim[0] = transform(inProj, outProj, lonlim[1], latlim[0])        
        
    else:
        Projection = 4326
        
    if level == "L3":
        Parameter = Parameter.format(AREA = Area)
        
    # Loop over the dates
    for Date_end in Dates_end:
        
        # Set the required time period
        if time_steps == "D":
            Start_day_payload = Date_end.day
            End_year_payload = Date_end.year  
            if Start_day_payload == 21:
                End_day_payload = 1
                End_month_payload = Date_end.month + 1
                if End_month_payload == 13:
                    End_month_payload = 1
                    End_year_payload += 1
            else:
                End_day_payload = Start_day_payload + 10
                End_month_payload = Date_end.month 
                                
        if time_steps == "M":
            Start_day_payload = Date_end.day
            End_day_payload = 1
            End_month_payload = Date_end.month + 1
            End_year_payload = Date_end.year  
            if End_month_payload == 13:
                End_month_payload = 1
                End_year_payload += 1
            
        if time_steps == "A":
            Start_day_payload = Date_end.day
            End_day_payload = 1
            End_month_payload = 1
            End_year_payload = Date_end.year + 1        

        file_name_temp = os.path.join(output_folder_para, "%s_WAPOR_%s_%s.%02d.%02d.tif" %(Parameter, dimension, Date_end.year, Date_end.month, Start_day_payload))
    
        if not os.path.exists(file_name_temp):   
            
            # Create payload file
            payload = Create_Payload_JSON(Parameter, Date_end, Start_day_payload, End_year_payload, End_month_payload, End_day_payload, latlim, lonlim, version, dimension, measure, Projection)
            success = 0
            no_succes = 0    
            
            while success == 0 and no_succes<10:
                
                try:
                    # Collect the date by using the payload file
                    response = requests.post(url, data=json.dumps(payload), headers=header)
                    response.raise_for_status()       
                    
                    response_json = response.json()
                    result = response_json['response']
                    
                    job_url = result['links'][0]['href']
                    
                    # output filename
                    print("Try to create %s" %file_name_temp)
                    
                    time.sleep(10)   
                    job_response = requests.get(job_url, headers=header)
                    if job_response.status_code == 200:
                        while job_response.json()['response']['status'] == 'RUNNING':
                            time.sleep(30)
                            job_response = requests.get(job_url, headers=header)
                        
                        if job_response.json()['response']['status'] == 'COMPLETED':
                            job_result = job_response.json()['response']['output']['downloadUrl']
                            urllib.request.urlretrieve(job_result, file_name_temp) 
                            print("Created %s succesfully!!!" %file_name_temp)
                            success = 1
                        else:
                            print("ERROR: Was not able to create output")  

                    else:
                        print("ERROR: Was not able to connect to WAPOR server")
                except:
                    success = 0
                    no_succes +=1
                    if no_succes == 10:
                        print("ERROR: already tried 10 times, and no connection with server. Please run code again")
    return()


def Create_Payload_JSON(Parameter, Date_end, Start_day_payload, End_year_payload, End_month_payload, End_day_payload, latlim, lonlim, version, dimension, measure, Projection):
    
    payload = {
      "type": "CropRaster",
      "params": {
        "properties": {
          "outputFileName": "%s_clipped.tif" %Parameter,
          "cutline": True,
          "tiled": True,
          "compressed": True,
          "overviews": True
        },
        "cube": {
          "code": Parameter,
          "workspaceCode": version,
          "language": "en"
        },
        "dimensions": [
          {
            "code": dimension,
            "values": [
              "[%d-%02d-%02d,%d-%02d-%02d)" %(Date_end.year, Date_end.month, Start_day_payload, End_year_payload, End_month_payload, End_day_payload)
            ]
          }
        ],
        "measures": [
          measure
        ],
        "shape": {
          "crs": "EPSG:%d" %Projection,
          "type": "Polygon",
          "coordinates": [
            [
              [
                lonlim[0],
                latlim[1]
              ],
              [
                lonlim[1],
                latlim[1]
              ],
              [
                lonlim[1],
                latlim[0]
              ],
              [
                lonlim[0],
                latlim[0]
              ],
              [
                lonlim[0],
                latlim[1]
              ]
            ]
          ]
        }
      }
    }
        
    return(payload)

class VariablesInfo:
    
    """
    This class contains the information about the WAPOR variables
    """
    
    descriptions = {'L1_GBWP_A': 'Gross Biomass Water Productivity',
             'L1_NBWP_A': 'Net Biomass Water Productivity',
             'L1_AETI_A': 'Actual EvapoTranspiration and Interception (Annual)',
             'L1_AETI_M': 'Actual EvapoTranspiration and Interception (Monthly)',
             'L1_AETI_D': 'Actual EvapoTranspiration and Interception (Dekadal)',
             'L1_T_A': 'Transpiration (Annual)',
             'L1_E_A': 'Evaporation (Annual)',
             'L1_I_A': 'Interception (Annual)',
             'L1_T_D': 'Transpiration (Dekadal)',
             'L1_E_D': 'Evaporation (Dekadal)',
             'L1_I_D': 'Interception (Dekadal)',
             'L1_NPP_D': 'Net Primary Production',
             'L1_TBP_A': 'Total Biomass Production (Annual)',
             'L1_LCC_A': 'Land Cover Classification',
             'L1_RET_A': 'Reference EvapoTranspiration (Annual)',
             'L1_PCP_A': 'Precipitation (Annual)',
             'L1_RET_M': 'Reference EvapoTranspiration (Monthly)',
             'L1_PCP_M': 'Precipitation (Monthly)',
             'L1_RET_D': 'Reference EvapoTranspiration (Dekadal)',
             'L1_PCP_D': 'Precipitation (Dekadal)',
             'L1_RET_E': 'Reference EvapoTranspiration (Daily)',
             'L1_PCP_E': 'Precipitation (Daily)',
             'L1_QUAL_NDVI_D': 'Quality of Normalized Difference Vegetation Index (Dekadal)',
             'L1_QUAL_LST_D': 'Quality Land Surface Temperature (Dekadal)',
              # 'L2_GBWP_S': 'Gross Biomass Water Productivity (Seasonal)',
             'L2_AETI_A': 'Actual EvapoTranspiration and Interception (Annual)',
             'L2_AETI_M': 'Actual EvapoTranspiration and Interception (Monthly)',
             'L2_AETI_D': 'Actual EvapoTranspiration and Interception (Dekadal)',
             'L2_T_A': 'Transpiration (Annual)',
             'L2_E_A': 'Evaporation (Annual)',
             'L2_I_A': 'Interception (Annual)',
             'L2_T_D': 'Transpiration (Dekadal)',
             'L2_E_D': 'Evaporation (Dekadal)',
             'L2_I_D': 'Interception (Dekadal)',
             'L2_NPP_D': 'Net Primary Production',
              #'L2_TBP_S': 'Total Biomass Production (Seasonal)',
             'L2_LCC_A': 'Land Cover Classification',
              #'L2_PHE_S': 'Phenology (Seasonal)',
             'L2_QUAL_NDVI_D': 'Quality of Normalized Difference Vegetation Index (Dekadal)',
             'L2_QUAL_LST_D': 'Quality Land Surface Temperature (Dekadal)',
             'L3_AETI_A': 'Actual EvapoTranspiration and Interception (Annual)',
             'L3_AETI_M': 'Actual EvapoTranspiration and Interception (Monthly)',
             'L3_AETI_D': 'Actual EvapoTranspiration and Interception (Dekadal)',
             'L3_T_A': 'Transpiration (Annual)',
             'L3_E_A': 'Evaporation (Annual)',
             'L3_I_A': 'Interception (Annual)',
             'L3_T_D': 'Transpiration (Dekadal)',
             'L3_E_D': 'Evaporation (Dekadal)',
             'L3_I_D': 'Interception (Dekadal)',
             'L3_NPP_D': 'Net Primary Production',
             'L3_QUAL_NDVI_D': 'Quality of Normalized Difference Vegetation Index (Dekadal)',
             'L3_QUAL_LST_D': 'Quality Land Surface Temperature (Dekadal)',
             'L3_LCC_A': 'Land Cover Classification'}
    
    measures = {'L1_GBWP_A': 'WPR',
             'L1_NBWP_A': 'WPR',
             'L1_AETI_A': 'WATER_MM',
             'L1_AETI_M': 'WATER_MM',
             'L1_AETI_D': 'WATER_MM',
             'L1_T_A': 'WATER_MM',
             'L1_E_A': 'WATER_MM',
             'L1_I_A': 'WATER_MM',
             'L1_T_D': 'WATER_MM',
             'L1_E_D': 'WATER_MM',
             'L1_I_D': 'WATER_MM',
             'L1_NPP_D': 'NPP',
             'L1_TBP_A': 'LPR',
             'L1_LCC_A': 'LCC',
             'L1_RET_A': 'WATER_MM',
             'L1_PCP_A': 'WATER_MM',
             'L1_RET_M': 'WATER_MM',
             'L1_PCP_M': 'WATER_MM',
             'L1_RET_D': 'WATER_MM',
             'L1_PCP_D': 'WATER_MM',
             'L1_RET_E': 'WATER_MM',
             'L1_PCP_E': 'WATER_MM',
             'L1_QUAL_NDVI_D': 'N_DAYS',
             'L1_QUAL_LST_D': 'N_DAYS',
              # 'L2_GBWP_S': 'WPR',
             'L2_AETI_A': 'WATER_MM',
             'L2_AETI_M': 'WATER_MM',
             'L2_AETI_D': 'WATER_MM',
             'L2_T_A': 'WATER_MM',
             'L2_E_A': 'WATER_MM',
             'L2_I_A': 'WATER_MM',
             'L2_T_D': 'WATER_MM',
             'L2_E_D': 'WATER_MM',
             'L2_I_D': 'WATER_MM',
             'L2_NPP_D': 'NPP',
              # 'L2_TBP_S': 'LPR',
             'L2_LCC_A': 'LCC',
              # 'L2_PHE_S': 'PHE',
             'L2_QUAL_NDVI_D': 'N_DAYS',
             'L2_QUAL_LST_D': 'N_DAYS',
             'L3_AETI_{AREA}_A': 'WATER_MM',
             'L3_AETI_{AREA}_M': 'WATER_MM',
             'L3_AETI_{AREA}_D': 'WATER_MM',
             'L3_T_{AREA}_A': 'WATER_MM',
             'L3_E_{AREA}_A': 'WATER_MM',
             'L3_I_{AREA}_A': 'WATER_MM',
             'L3_T_{AREA}_D': 'WATER_MM',
             'L3_E_{AREA}_D': 'WATER_MM',
             'L3_I_{AREA}_D': 'WATER_MM',
             'L3_NPP_{AREA}_D': 'NPP',
             'L3_QUAL_NDVI_{AREA}_D': 'N_DAYS',
             'L3_QUAL_LST_{AREA}_D': 'N_DAYS',
             'L3_LCC_{AREA}_A': 'LCC'}
    
    dimensions = {'L1_GBWP_A': 'YEAR',
             'L1_NBWP_A': 'YEAR',
             'L1_AETI_A': 'YEAR',
             'L1_AETI_M': 'MONTH',
             'L1_AETI_D': 'DEKAD',
             'L1_T_A': 'YEAR',
             'L1_E_A': 'YEAR',
             'L1_I_A': 'YEAR',
             'L1_T_D': 'DEKAD',
             'L1_E_D': 'DEKAD',
             'L1_I_D': 'DEKAD',
             'L1_NPP_D': 'DEKAD',
             'L1_TBP_A': 'YEAR',
             'L1_LCC_A': 'YEAR',
             'L1_RET_A': 'YEAR',
             'L1_PCP_A': 'YEAR',
             'L1_RET_M': 'MONTH',
             'L1_PCP_M': 'MONTH',
             'L1_RET_D': 'DEKAD',
             'L1_PCP_D': 'DEKAD',
             'L1_RET_E': 'DAY',
             'L1_PCP_E': 'DAY',
             'L1_QUAL_NDVI_D': 'DEKAD',
             'L1_QUAL_LST_D': 'DEKAD',
              # 'L2_GBWP_S': 'SEASON',
             'L2_AETI_A': 'YEAR',
             'L2_AETI_M': 'MONTH',
             'L2_AETI_D': 'DEKAD',
             'L2_T_A': 'YEAR',
             'L2_E_A': 'YEAR',
             'L2_I_A': 'YEAR',
             'L2_T_D': 'DEKAD',
             'L2_E_D': 'DEKAD',
             'L2_I_D': 'DEKAD',
             'L2_NPP_D': 'DEKAD',
              # 'L2_TBP_S': 'SEASON',
             'L2_LCC_A': 'YEAR',
              # 'L2_PHE_S': 'SEASON',
             'L2_QUAL_NDVI_D': 'DEKAD',
             'L2_QUAL_LST_D': 'DEKAD',
             'L3_AETI_{AREA}_A': 'YEAR',
             'L3_AETI_{AREA}_M': 'MONTH',
             'L3_AETI_{AREA}_D': 'DEKAD',
             'L3_T_{AREA}_A': 'YEAR',
             'L3_E_{AREA}_A': 'YEAR',
             'L3_I_{AREA}_A': 'YEAR',
             'L3_T_{AREA}_D': 'DEKAD',
             'L3_E_{AREA}_D': 'DEKAD',
             'L3_I_{AREA}_D': 'DEKAD',
             'L3_NPP_{AREA}_D': 'DEKAD',
             'L3_QUAL_NDVI_{AREA}_D': 'DEKAD',
             'L3_QUAL_LST_{AREA}_D': 'DEKAD',
             'L3_LCC_{AREA}_A': 'YEAR'}

    versions = {'1': 'WAPOR',
                '2': 'WAPOR_2'}
    
class LEVEL3:
    
    """
    This class contains the information of the areas in LEVEL 3
    """
    
    Area = {'AWA': 'Awash, Ethiopia',
             'BKA': 'Bekaa, Lebanon',
             'KOG': 'Koga, Ethiopia',
             'ODN': 'Office du Niger, Mali',
             'ZAN': 'Zankalon, Egypt'}
    
    Projection = {'AWA': 32637,
             'BKA': 32636,
             'KOG': 32637,
             'ODN': 32630,
             'ZAN': 32636}
    



# -*- coding: utf-8 -*-
"""
Created on Thu Oct 22 14:44:51 2020

@author: timhe
"""
import matplotlib.pyplot as plt
import numpy as np

def Show_array(Array):
    
    rng = Array.flatten()
    rng[rng==-9999] = np.nan
    rng = rng[~np.isnan(rng)]
    plt.hist(rng, bins='auto')  # arguments are passed to np.histogram
    plt.title("Histogram with 'auto' bins")
    plt.show()
    return()


# -*- coding: utf-8 -*-
"""
Created on Wed Sep  9 10:10:43 2020

@author: timhe
"""
import sys
from watertools.Collect.WAPOR.DataAccess import WAPOR

def main(output_folder, Startdate, Enddate, latlim, lonlim, Parameter, Area = None, Version = "2"):
    
    """
    This function downloads WAPOR data for the specified time
    interval, and spatial extent.

    Keyword arguments:
    -- output_folder: (string) to define the folder where to save the created tiff files.
    -- Startdate: (string) defines the startdate of the required dataset in the following format "yyyy-mm-dd"
    -- Enddate: (string) defines the enddate of the required dataset in the following format "yyyy-mm-dd"
    -- Latlim: (array) defines the latitude limits in the following format [Latitude_minimum, Latitude_maximum] e.g. [10, 13]
    -- Lonlim: (array) defines the longitude limits in the following format [Longitude_minimum, Longitude_maximum] e.g. [10, 13]
    -- API_KEY: (string) this is a private API KEY that can be collected here: https://io.apps.fao.org/gismgr/api/v1/swagger-ui.html#/IAM/apiKeySignIn
    -- Parameter: (string) define the parameter that is required (possible Parameters: watertools.Collect.WAPOR.DataAccess.VariablesInfo().descriptions.keys())
    -- version: (string) default is 2, but if version 1 is required use this parameter (options are "1" or "2")

    """
    
    print('\nDownload WAPOR %s data for period %s till %s' %(Parameter, Startdate, Enddate))
    WAPOR(output_folder, Startdate, Enddate, latlim, lonlim, Parameter, Area = None, Version = "2")

if __name__ == '__main__':
    main(sys.argv)   


# -*- coding: utf-8 -*-

from watertools import General, Collect, Products, Functions, Set_Up_watertools

__all__ = ['General', 'Collect', 'Products', 'Functions', 'Set_Up_watertools']

__version__ = '0.1'



# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/ALEXI

Restrictions:
The data and this python file may not be distributed to others without
permission of the WA+ team due data restriction of the ALEXI developers.

Description:
This script collects ALEXI data from the UNESCO-IHE FTP server. The data has a
monthly temporal resolution and a spatial resolution of 0.05 degree. The
resulting tiff files are in the WGS84 projection.
The data is available between 2003-01-01 till 2014-12-31.

Example:
from watertools.Collect import ALEXI
ALEXI.monthly(Dir='C:/Temp/', Startdate='2003-02-24', Enddate='2003-03-09',
                     latlim=[50,54], lonlim=[3,7])

"""
# General modules
import numpy as np
import os
import pandas as pd
from ftplib import FTP
import datetime
import math
import glob

# Water Accounting Modules
import watertools
import watertools.General.raster_conversions as RC
import watertools.General.data_conversions as DC

def DownloadData(Dir, Startdate, Enddate, latlim, lonlim, TimeStep, Waitbar):

    """
    This scripts downloads ALEXI ET data from the UNESCO-IHE ftp server.
    The output files display the total ET in mm for a period of one week.
    The name of the file corresponds to the first day of the week.

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    TimeStep -- 'daily' or 'weekly'  (by using here monthly, an older dataset will be used)
    lonlim -- [ymin, ymax] (values must be between -60 and 70)
    latlim -- [xmin, xmax] (values must be between -180 and 180)
    """
    # Check the latitude and longitude and otherwise set lat or lon on greatest extent
    if latlim[0] < -60 or latlim[1] > 70:
        print('Latitude above 70N or below 60S is not possible. Value set to maximum')
        latlim[0] = np.max(latlim[0],-60)
        latlim[1] = np.min(latlim[1],70)
    if lonlim[0] < -180 or lonlim[1] > 180:
        print ('Longitude must be between 180E and 180W. Now value is set to maximum')
        lonlim[0] = np.max(lonlim[0],-180)
        lonlim[1] = np.min(lonlim[1],180)

	# Check Startdate and Enddate
    if not Startdate:
        if TimeStep == 'weekly':
            Startdate = pd.Timestamp('2003-01-01')
        if TimeStep == 'daily':
            Startdate = pd.Timestamp('2005-01-01')
    if not Enddate:
        if TimeStep == 'weekly':
            Enddate = pd.Timestamp('2015-12-31')
        if TimeStep == 'daily':
            Enddate = pd.Timestamp('2016-12-31')

	# Make a panda timestamp of the date
    try:
        Enddate = pd.Timestamp(Enddate)
    except:
        Enddate = Enddate

    if TimeStep == 'weekly':

        # Define the Startdate of ALEXI
        DOY = datetime.datetime.strptime(Startdate,
                                         '%Y-%m-%d').timetuple().tm_yday
        Year = datetime.datetime.strptime(Startdate,
                                          '%Y-%m-%d').timetuple().tm_year

      	# Change the startdate so it includes an ALEXI date
        DOYstart = int(math.ceil(DOY/7.0)*7+1)
        DOYstart = str('%s-%s' %(DOYstart, Year))
        Day = datetime.datetime.strptime(DOYstart, '%j-%Y')
        Month = '%02d' % Day.month
        Day = '%02d' % Day.day
        Date = (str(Year) + '-' + str(Month) + '-' + str(Day))
        DOY = datetime.datetime.strptime(Date,
                                         '%Y-%m-%d').timetuple().tm_yday
        # The new Startdate
        Date = pd.Timestamp(Date)

        # amount of Dates weekly
        Dates = pd.date_range(Date, Enddate, freq = '7D')

        # Define directory and create it if not exists
        output_folder = os.path.join(Dir, 'Evaporation', 'ALEXI', 'Weekly')
        if not os.path.exists(output_folder):
            os.makedirs(output_folder)

    if TimeStep == 'daily':

        # Define Dates
        Dates = pd.date_range(Startdate, Enddate, freq = 'D')

        # Define directory and create it if not exists
        output_folder = os.path.join(Dir, 'Evaporation', 'ALEXI', 'Daily')
        if not os.path.exists(output_folder):
            os.makedirs(output_folder)

    # Create Waitbar
    total_amount = len(Dates)
    if Waitbar == 1:
        import watertools.Functions.Random.WaitbarConsole as WaitbarConsole
        amount = 0
        WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)

    if TimeStep == 'weekly':
        ALEXI_weekly(Date, Enddate, output_folder, latlim, lonlim, Year, Waitbar, total_amount, TimeStep)

    if TimeStep == 'daily':
        ALEXI_daily(Dates, output_folder, latlim, lonlim, Waitbar, total_amount, TimeStep)

def ALEXI_weekly(Date, Enddate, output_folder, latlim, lonlim, Year, Waitbar, total_amount, TimeStep):

    # Define the stop conditions
    Stop = Enddate.toordinal()
    End_date=0
    amount = 0
    while End_date == 0:

        # Date as printed in filename
        Datesname=Date+pd.DateOffset(days=-7)
        DirFile= os.path.join(output_folder,'ETa_ALEXI_CSFR_mm-week-1_weekly_%s.%02s.%02s.tif' %(Datesname.strftime('%Y'), Datesname.strftime('%m'), Datesname.strftime('%d')))

        # Define end filename
        filename = "ALEXI_weekly_mm_%s_%s.tif" %(Date.strftime('%j'), Date.strftime('%Y'))

		 # Temporary filename for the downloaded global file
        local_filename = os.path.join(output_folder, filename)

	    # Create the new date for the next download
        Datename = (str(Date.strftime('%Y')) + '-' + str(Date.strftime('%m')) + '-' + str(Date.strftime('%d')))

        # Define IDs
        yID = 3000 - np.int16(np.array([np.ceil((latlim[1]+60)*20),np.floor((latlim[0]+60)*20)]))
        xID = np.int16(np.array([np.floor((lonlim[0])*20),np.ceil((lonlim[1])*20)])+3600)

        # Download the data from FTP server if the file not exists
        if not os.path.exists(DirFile):
            try:
                Download_ALEXI_from_WA_FTP(local_filename, DirFile, filename, lonlim, latlim, yID, xID, TimeStep)
            except:
                print("Was not able to download file with date %s" %Date)

        # Current DOY
        DOY = datetime.datetime.strptime(Datename,
                                     '%Y-%m-%d').timetuple().tm_yday

        # Define next day
        DOY_next = int(DOY + 7)
        if DOY_next >= 366:
            DOY_next = 8
            Year += 1
        DOYnext = str('%s-%s' %(DOY_next, Year))
        DayNext = datetime.datetime.strptime(DOYnext, '%j-%Y')
        Month = '%02d' % DayNext.month
        Day = '%02d' % DayNext.day
        Date = (str(Year) + '-' + str(Month) + '-' + str(Day))

        # Adjust waitbar
        if Waitbar == 1:
            import watertools.Functions.Random.WaitbarConsole as WaitbarConsole
            amount += 1
            WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)

        # Check if this file must be downloaded
        Date = pd.Timestamp(Date)
        if Date.toordinal() > Stop:
            End_date = 1

def ALEXI_daily(Dates, output_folder, latlim, lonlim, Waitbar, total_amount, TimeStep):

    amount = 0
    for Date in Dates:

        # Date as printed in filename
        DirFile= os.path.join(output_folder,'ETa_ALEXI_CSFR_mm-day-1_daily_%d.%02d.%02d.tif' %(Date.year, Date.month, Date.day))
        DOY = Date.timetuple().tm_yday

        # Define end filename
        filename = "EDAY_CERES_%d%03d.dat.gz" %(Date.year, DOY)

		 # Temporary filename for the downloaded global file
        local_filename = os.path.join(output_folder, filename)

        # Define IDs
        yID = 3000 - np.int16(np.array([np.ceil((latlim[1]+60)*20),np.floor((latlim[0]+60)*20)]))
        xID = np.int16(np.array([np.floor((lonlim[0])*20),np.ceil((lonlim[1])*20)])+3600)

        # Download the data from FTP server if the file not exists
        if not os.path.exists(DirFile):
            try:
                Download_ALEXI_from_WA_FTP(local_filename, DirFile, filename, lonlim, latlim, yID, xID, TimeStep)
            except:
                print("Was not able to download file with date %s" %Date)

        # Adjust waitbar
        if Waitbar == 1:
            import watertools.Functions.Random.WaitbarConsole as WaitbarConsole
            amount += 1
            WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)

    os.chdir(output_folder)
    re = glob.glob("*.dat")
    for f in re:
        os.remove(os.path.join(output_folder, f))

def Download_ALEXI_from_WA_FTP(local_filename, DirFile, filename, lonlim, latlim, yID, xID, TimeStep):
    """
    This function retrieves ALEXI data for a given date from the
    ftp.wateraccounting.unesco-ihe.org server.

    Restrictions:
    The data and this python file may not be distributed to others without
    permission of the WA+ team due data restriction of the ALEXI developers.

    Keyword arguments:
	local_filename -- name of the temporary file which contains global ALEXI data
    DirFile -- name of the end file with the weekly ALEXI data
    filename -- name of the end file
    lonlim -- [ymin, ymax] (values must be between -60 and 70)
    latlim -- [xmin, xmax] (values must be between -180 and 180)
    """

    # Collect account and FTP information
    username, password = watertools.Functions.Random.Get_Username_PWD.GET('FTP_WA')
    ftpserver = "ftp.wateraccounting.unesco-ihe.org"

    # Download data from FTP
    ftp=FTP(ftpserver)
    ftp.login(username,password)
    if TimeStep == "weekly":
        directory="/WaterAccounting/Data_Satellite/Evaporation/ALEXI/World/"
    if TimeStep == "daily":
        directory="/WaterAccounting/Data_Satellite/Evaporation/ALEXI/World_05182018/"
    ftp.cwd(directory)
    lf = open(local_filename, "wb")
    ftp.retrbinary("RETR " + filename, lf.write)
    lf.close()

    if TimeStep == "weekly":

        # Open global ALEXI data
        dataset = RC.Open_tiff_array(local_filename)

        # Clip extend out of world data
        data = dataset[yID[0]:yID[1],xID[0]:xID[1]]
        data[data < 0] = -9999

    if TimeStep == "daily":

        DC.Extract_Data_gz(local_filename, os.path.splitext(local_filename)[0])

        raw_data = np.fromfile(os.path.splitext(local_filename)[0], dtype="<f4")
        dataset = np.flipud(np.resize(raw_data, [3000,7200]))
        data = dataset[yID[0]:yID[1],xID[0]:xID[1]] / 2.45                      # Values are in MJ/m2d so convert to mm/d
        data[data < 0] = -9999

    # make geotiff file
    geo = [lonlim[0],0.05,0,latlim[1],0,-0.05]
    DC.Save_as_tiff(name = DirFile, data = data, geo = geo, projection = "WGS84")
    return



# -*- coding: utf-8 -*-
'''
Authors: Tim Hessels
Module: Products/ETref
'''
# import general python modules
import sys
import numpy as np

# import watertools modules
from watertools.Products.ETref.CollectDataETref import CollectData
from watertools.Products.ETref.CollectLANDSAFETref import CollectLANDSAF
from watertools.Products.ETref.SetVarETref import SetVariables

def main(Dir, Startdate = '', Enddate = '',
         latlim = [-60, 60], lonlim = [-180, 180], pixel_size = False, cores = False, LANDSAF =  0, SourceLANDSAF=  '', Waitbar = 1):
    """
    This function creates ETref (daily) data based on Hydroshed, GLDAS, and (CFSR/LANDSAF)

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    latlim -- [ymin, ymax] (values must be between -60 and 60)
    lonlim -- [xmin, xmax] (values must be between -180 and 180)
    pixel_size -- The output pixel size
    cores -- The number of cores used to run the routine.
             It can be 'False' to avoid using parallel computing
             routines.
    LANDSAF -- if LANDSAF data must be used it is 1
    SourceLANDSAF -- the path to the LANDSAF files
    Waitbar -- 1 (Default) will print the waitbar
    """
    if Waitbar == 1:
        print('Create daily Reference ET data for the period %s till %s' %(Startdate, Enddate))

    # Correct latitude and longitude if needed
    if latlim[0] < -60 or latlim[1] > 60:
        print('Latitude above 60N or below 60S is not possible.'
                        ' Value set to maximum')
        latlim[0] = np.max(latlim[0], -60)
        latlim[1] = np.min(lonlim[1], 60)
    if lonlim[0] < -180 or lonlim[1] > 180:
        print('Longitude must be between 180E and 180W.'
                       ' Now value is set to maximum')
        lonlim[0] = np.max(latlim[0], -180)
        lonlim[1] = np.min(lonlim[1], 180)

    # Download data (using the wa.Collect scripts)
    CollectData(Dir, Startdate, Enddate, latlim, lonlim, cores, LANDSAF)

    # Process LANDSAF data if needed
    if LANDSAF == 1:
        CollectLANDSAF(SourceLANDSAF, Dir, Startdate, Enddate, latlim, lonlim)

    # Set up the variables and calculates ETref hereafter
    SetVariables(Dir, Startdate, Enddate, latlim, lonlim, pixel_size, cores, LANDSAF, Waitbar)


if __name__ == '__main__':
    main(sys.argv)



# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/GLEAM

Description:
This module downloads GLEAM ET data from
www.gleam.eu. Use the GLEAM.ET_monthly function to
download and create monthly ET images in Gtiff format.

Examples:
from watertools.Collect import GLEAM
GLEAM.ET_monthly(Dir='C:/Temp3/', Startdate='2003-12-01', Enddate='2004-01-20',
           latlim=[20,30], lonlim=[30, 40])
from watertools.Collect import GLEAM
GLEAM.ET_daily(Dir='C:/Temp3/', Startdate='2003-12-01', Enddate='2004-01-20',
           latlim=[20,30], lonlim=[30, 40])

!!!!
You need a password to download this data. To get a password, you have to make an account here:  http://www.gleam.eu/#4
!!!!

"""

from .ET_monthly import main as ET_monthly
from .ET_daily import main as ET_daily
from .ETpot_monthly import main as ETpot_monthly
from .ETpot_daily import main as ETpot_daily

__all__ = ['ET_monthly', 'ET_daily', 'ETpot_monthly', 'ETpot_daily']

__version__ = '0.1'



# -*- coding: utf-8 -*-
'''
Authors: Tim Hessels
Module: Products/SoilGrids
'''

# import general python modules
import os
from osgeo import gdal, osr
import numpy as np
import pandas as pd
import subprocess
import netCDF4
import glob

# import watertools modules
from watertools.General import data_conversions as DC
from watertools.General import raster_conversions as RC

def Topsoil(Dir, latlim, lonlim):
    """
    This function calculates the topsoil Water Holding Capacity soil characteristic (15cm)

    Keyword arguments:
    Dir -- 'C:/' path to the map
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    """
	
    print('/nCreate Water Holding Capacity map of the topsoil from SoilGrids')
    
    # Define parameters to define the topsoil
    SL = "sl3"
	
    Calc_Property(Dir, latlim, lonlim, SL)
	
    return

def Subsoil(Dir, latlim, lonlim):
    """
    This function calculates the subsoil Water Holding Capacity soil characteristic (100cm)

    Keyword arguments:
    Dir -- 'C:/' path to the WA map
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    """
	
    print('/nCreate Water Holding Capacity map of the subsoil from SoilGrids')
    
	 # Define parameters to define the subsoil	
    SL = "sl6"
	
    Calc_Property(Dir, latlim, lonlim, SL)
	
    return
	
def Calc_Property(Dir, latlim, lonlim, SL):	
    
    import watertools
    
    # Define level
    if SL == "sl3":
        level = "Topsoil"
    elif SL == "sl6":
        level = "Subsoil" 
    
    # check if you need to download
    filename_out_thetasat = os.path.join(Dir, 'SoilGrids', 'Theta_Sat' ,'Theta_Sat_%s_SoilGrids_kg-kg.tif' %level)
    if not os.path.exists(filename_out_thetasat):
       if SL == "sl3":
           watertools.Products.SoilGrids.Theta_Sat.Topsoil(Dir, latlim, lonlim)
       elif SL == "sl6":
           watertools.Products.SoilGrids.Theta_Sat.Subsoil(Dir, latlim, lonlim)

    filedir_out_whc = os.path.join(Dir, 'SoilGrids', 'Water_Holding_Capacity')
    if not os.path.exists(filedir_out_whc):
        os.makedirs(filedir_out_whc)    
               
    # Define theta field capacity output
    filename_out_whc = os.path.join(filedir_out_whc, 'Water_Holding_Capacity_%s_SoilGrids_mm-m.tif' %level)
    
    if not os.path.exists(filename_out_whc):
        
        # Get info layer
        geo_out, proj, size_X, size_Y = RC.Open_array_info(filename_out_thetasat)

        # Open dataset
        theta_sat = RC.Open_tiff_array(filename_out_thetasat)
        
        # Calculate theta field capacity
        theta_whc = np.ones(theta_sat.shape) * -9999   
        theta_whc = np.where(theta_sat < 0.301, 80, 450 * np.arccosh(theta_sat + 0.7) - 2 * (theta_sat + 0.7) + 20)        
               
        # Save as tiff
        DC.Save_as_tiff(filename_out_whc, theta_whc, geo_out, proj)
    return           
           
           
	
	





# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/SSEBop

Restrictions:
The data and this python file may not be distributed to others without
permission of the WA+ team due data restriction of the SSEBop developers.

Description:
This script collects SSEBop data from the UNESCO-IHE FTP server or from the web. The data has a
monthly temporal resolution and a spatial resolution of 0.01 degree. The
resulting tiff files are in the WGS84 projection.
The data is available between 2003-01-01 till present.

Example:
from watertools.Collect import SSEBop
SSEBop.monthly(Dir='C:/Temp/', Startdate='2003-02-24', Enddate='2003-03-09',
                     latlim=[50,54], lonlim=[3,7])

"""
# General modules
import numpy as np
import os
import pandas as pd
from ftplib import FTP
import sys
if sys.version_info[0] == 3:
    import urllib.parse
if sys.version_info[0] == 2:
    import urllib
# Water Accounting Modules
import watertools
import watertools.General.raster_conversions as RC
import watertools.General.data_conversions as DC


def DownloadData(Dir, Startdate, Enddate, latlim, lonlim, Waitbar, version, Product):
    """
    This scripts downloads SSEBop ET data from the UNESCO-IHE ftp server.
    The output files display the total ET in mm for a period of one month.
    The name of the file corresponds to the first day of the month.

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    lonlim -- [ymin, ymax] (values must be between -90 and 90)
    latlim -- [xmin, xmax] (values must be between -180 and 180)
    """

    if version == "FTP":
        # Check the latitude and longitude and otherwise set lat or lon on greatest extent
        if latlim[0] < -59.2 or latlim[1] > 80:
            print('Latitude above 80N or below -59.2S is not possible. Value set to maximum')
            latlim[0] = np.max(latlim[0], -59.2)
            latlim[1] = np.min(latlim[1], 80)
        if lonlim[0] < -180 or lonlim[1] > 180:
            print('Longitude must be between 180E and 180W. Now value is set to maximum')
            lonlim[0] = np.max(lonlim[0],-180)
            lonlim[1] = np.min(lonlim[1],180)

    	# Check Startdate and Enddate
        if not Startdate:
            Startdate = pd.Timestamp('2003-01-01')
        if not Enddate:
            Enddate = pd.Timestamp('2014-10-31')

    if version == "V4":
        # Check the latitude and longitude and otherwise set lat or lon on greatest extent
        if latlim[0] < -60 or latlim[1] > 80.0022588483988670:
            print('Latitude above 80N or below -59.2S is not possible. Value set to maximum')
            latlim[0] = np.max(latlim[0], -60)
            latlim[1] = np.min(latlim[1], 80.0022588483988670)
        if lonlim[0] < -180 or lonlim[1] > 180.0002930387853439:
            print('Longitude must be between 180E and 180W. Now value is set to maximum')
            lonlim[0] = np.max(lonlim[0],-180)
            lonlim[1] = np.min(lonlim[1],180.0002930387853439)

    	# Check Startdate and Enddate
        if not Startdate:
            Startdate = pd.Timestamp('2003-01-01')
        if not Enddate:
            import datetime
            Enddate = pd.Timestamp(datetime.datetime.now())

   # Define directory and create it if not exists
    if Product == "ETact":
        output_folder = os.path.join(Dir, 'Evaporation', 'SSEBop', 'Monthly')
        freq_use = "MS"
    if Product == "ETpot":
        output_folder = os.path.join(Dir, 'Potential_Evapotranspiration', 'FEWS', 'Daily')        
        freq_use = "D"

    if not os.path.exists(output_folder):
        os.makedirs(output_folder)

    # Creates dates library
    Dates = pd.date_range(Startdate, Enddate, freq = freq_use)

    # Create Waitbar
    if Waitbar == 1:
        import watertools.Functions.Random.WaitbarConsole as WaitbarConsole
        total_amount = len(Dates)
        amount = 0
        WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)

    # Loop over the dates
    for Date in Dates:

        # Define year and month
        year = Date.year
        month = Date.month
        day = Date.day
        
        if version == "FTP":

            # Date as printed in filename
            Filename_out= os.path.join(output_folder,'ETa_SSEBop_FTP_mm-month-1_monthly_%s.%02s.%02s.tif' %(Date.strftime('%Y'), Date.strftime('%m'), Date.strftime('%d')))

            # Define end filename
            Filename_dir = os.path.join("%s" %year, "m%s%02d.tif" %(str(year)[2:], month))
            Filename_only = "m%s%02d.tif" %(str(year)[2:], month)

        if version == "V4":

            # Date as printed in filename
            if Product == "ETpot":
                Filename_out= os.path.join(output_folder,'ETpot_FEWS_mm-day-1_daily_%s.%02s.%02s.tif' %(Date.strftime('%Y'), Date.strftime('%m'), Date.strftime('%d')))
                # Define the downloaded zip file
                Filename_only_zip = 'et%02s%02d%02d.tar.gz' %(str(year)[2:], month, day)
                # The end file name after downloading and unzipping
                Filename_only = "et%02s%02d%02d.bil" %(str(year)[2:], month, day)
                # Create bin folder
                temp_folder = os.path.join(output_folder, "Temp")
                if not os.path.exists(temp_folder):
                    os.makedirs(temp_folder)
                local_filename = os.path.join(temp_folder, Filename_only)

            if Product == "ETact":
                Filename_out= os.path.join(output_folder,'ETa_SSEBop_V4_mm-month-1_monthly_%s.%02s.%02s.tif' %(Date.strftime('%Y'), Date.strftime('%m'), Date.strftime('%d')))
                # Define the downloaded zip file
                Filename_only_zip = "m%s%02d.zip" %(str(year), month)
                # The end file name after downloading and unzipping
                Filename_only = "m%s%02d_modisSSEBopETv4_actual_mm.tif" %(str(year), month)

        		    # Temporary filename for the downloaded global file
                local_filename = os.path.join(output_folder, Filename_only)

        # Download the data from FTP server if the file not exists
        if not os.path.exists(Filename_out):
            try:

                if version == "FTP":
                    Download_SSEBop_from_WA_FTP(local_filename, Filename_dir)
                if version == "V4":
                    if Product == "ETpot":
                        Download_SSEBop_from_Web(temp_folder, Filename_only_zip, Product)
                    if Product == "ETact":
                        Download_SSEBop_from_Web(output_folder, Filename_only_zip, Product)
                        
                if Product == "ETpot":
                    Array_ETpot = RC.Open_bil_array(local_filename)
                    Array_ETpot = Array_ETpot/100
                    Geo_out = tuple([-180.5, 1, 0, 90.5, 0, -1])
                    dest = DC.Save_as_MEM(Array_ETpot, Geo_out, "WGS84")
                    data, Geo_out, proj = RC.clip_data(dest, latlim, lonlim)
                    DC.Save_as_tiff(Filename_out, data, Geo_out, "WGS84")
                    
                if Product == "ETact":    
                    # Clip dataset
                    data, Geo_out, proj = RC.clip_data(local_filename, latlim, lonlim)
                    data[data<-9999] = -9999
                    DC.Save_as_tiff(Filename_out, data, Geo_out, "WGS84")
                    os.remove(local_filename)

            except:
                print("Was not able to download file with date %s" %Date)

        # Adjust waitbar
        if Waitbar == 1:
            amount += 1
            WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)

    if version == "V4":
        import glob
        os.chdir(output_folder)
        if Product == "ETact":
            zipfiles = glob.glob("*.zip")
            for zipfile in zipfiles:
                os.remove(os.path.join(output_folder, zipfile))
            xmlfiles = glob.glob("*.xml")
            for xmlfile in xmlfiles:
                os.remove(os.path.join(output_folder, xmlfile))
        if Product == "ETpot":  
            import shutil
            Temp_dir = os.path.join(output_folder, "Temp")
            shutil.rmtree(Temp_dir)
            
    return

def Download_SSEBop_from_WA_FTP(local_filename, Filename_dir):
    """
    This function retrieves SSEBop data for a given date from the
    ftp.wateraccounting.unesco-ihe.org server.

    Restrictions:
    The data and this python file may not be distributed to others without
    permission of the WA+ team due data restriction of the SSEBop developers.

    Keyword arguments:
	 local_filename -- name of the temporary file which contains global SSEBop data
    Filename_dir -- name of the end file with the monthly SSEBop data
    """

    # Collect account and FTP information
    username, password = watertools.Functions.Random.Get_Username_PWD.GET("FTP_WA")
    ftpserver = "ftp.wateraccounting.unesco-ihe.org"

    # Download data from FTP
    ftp=FTP(ftpserver)
    ftp.login(username,password)
    directory="/WaterAccounting/Data_Satellite/Evaporation/SSEBop/sourcefiles/"
    ftp.cwd(directory)
    lf = open(local_filename, "wb")
    ftp.retrbinary("RETR " + Filename_dir, lf.write)
    lf.close()

    return

def Download_SSEBop_from_Web(output_folder, Filename_only_zip, Product):
    """
    This function retrieves SSEBop data for a given date from the
    https://edcintl.cr.usgs.gov server.

    Keyword arguments:
	 local_filename -- name of the temporary file which contains global SSEBop data
    Filename_dir -- name of the end directory to put in the extracted data
    """
    if Product == "ETact":
        # Create the total url to the webpage
        total_URL = "https://edcintl.cr.usgs.gov/downloads/sciweb1/shared/fews/web/global/monthly/eta/downloads/" + str(Filename_only_zip)

    if Product == "ETpot":
        total_URL = "https://edcintl.cr.usgs.gov/downloads/sciweb1/shared/fews/web/global/daily/pet/downloads/daily/" + str(Filename_only_zip)

    # Download the data
    if sys.version_info[0] == 2:
        urllib.urlretrieve(total_URL, os.path.join(output_folder, Filename_only_zip))
    if sys.version_info[0] == 3:
        urllib.request.urlretrieve(total_URL, os.path.join(output_folder, Filename_only_zip)) 

    # unzip the file
    if Product == "ETpot":
        DC.Extract_Data_tar_gz(os.path.join(output_folder, Filename_only_zip), output_folder)
    if Product == "ETact":
        DC.Extract_Data(os.path.join(output_folder, Filename_only_zip), output_folder)

    return




# -*- coding: utf-8 -*-
'''
Authors: Tim Hessels
Module: Products/ETref
'''
# import general python modules
import sys
import pandas as pd
import numpy as np
import calendar
import os
from osgeo import gdal

# import watertools modules
from watertools.General import raster_conversions as RC
from watertools.General import data_conversions as DC
from watertools.Products.ETref import daily

def main(Dir, Startdate = '', Enddate = '',
         latlim = [-60, 60], lonlim = [-180, 180], pixel_size = False, cores = False, LANDSAF =  0, SourceLANDSAF=  '', Waitbar = 1):
    """
    This function downloads TRMM3B43 V7 (monthly) data

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    latlim -- [ymin, ymax] (values must be between -50 and 50)
    lonlim -- [xmin, xmax] (values must be between -180 and 180)
    cores -- The number of cores used to run the routine.
             It can be 'False' to avoid using parallel computing
             routines.
    Waitbar -- 1 (Default) will print the waitbar
    """

    print('Create monthly Reference ET data for period %s till %s' %(Startdate, Enddate))

    # An array of monthly dates which will be calculated
    Dates = pd.date_range(Startdate,Enddate,freq = 'MS')

    # Create Waitbar
    if Waitbar == 1:
        import watertools.Functions.Random.WaitbarConsole as WaitbarConsole
        total_amount = len(Dates)
        amount = 0
        WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)

	# Calculate the ETref day by day for every month
    for Date in Dates:

        # Collect date data
        Y=Date.year
        M=Date.month
        Mday=calendar.monthrange(Y,M)[1]
        Days=pd.date_range(Date,Date+pd.Timedelta(days=Mday),freq='D')
        StartTime=Date.strftime('%Y')+'-'+Date.strftime('%m')+ '-01'
        EndTime=Date.strftime('%Y')+'-'+Date.strftime('%m')+'-'+str(Mday)

        # Get ETref on daily basis
        daily(Dir=Dir, Startdate=StartTime,Enddate=EndTime,latlim=latlim, lonlim=lonlim, pixel_size = pixel_size, cores=cores, LANDSAF=LANDSAF, SourceLANDSAF=SourceLANDSAF, Waitbar = 0)

        # Load DEM
        if not pixel_size:
            nameDEM='DEM_HydroShed_m_3s.tif'
            DEMmap=os.path.join(Dir,'HydroSHED','DEM',nameDEM )
        else:
            DEMmap=os.path.join(Dir,'HydroSHED','DEM','DEM_HydroShed_m_reshaped_for_ETref.tif')
        # Get some geo-data to save results
        geo_ET, proj, size_X, size_Y = RC.Open_array_info(DEMmap)

        dataMonth=np.zeros([size_Y,size_X])

        for Day in Days[:-1]:
            DirDay=os.path.join(Dir,'ETref','Daily','ETref_mm-day-1_daily_' + Day.strftime('%Y.%m.%d') + '.tif')
            dataDay=gdal.Open(DirDay)
            Dval=dataDay.GetRasterBand(1).ReadAsArray().astype(np.float32)
            Dval[Dval<0]=0
            dataMonth=dataMonth+Dval
            dataDay=None

        # make geotiff file
        output_folder_month=os.path.join(Dir,'ETref','Monthly')
        if os.path.exists(output_folder_month)==False:
            os.makedirs(output_folder_month)
        DirMonth=os.path.join(output_folder_month,'ETref_mm-month-1_monthly_'+Date.strftime('%Y.%m.%d') + '.tif')

        # Create the tiff file
        DC.Save_as_tiff(DirMonth,dataMonth, geo_ET, proj)

        # Create Waitbar
        if Waitbar == 1:
            amount += 1
            WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)


if __name__ == '__main__':
    main(sys.argv)



# -*- coding: utf-8 -*-
import sys
from watertools.Collect.CHIRPS.DataAccess import DownloadData


def main(Dir, Startdate='', Enddate='',
         latlim=[-50, 50], lonlim=[-180, 180], cores=False, Waitbar = 1):
                """
                This function downloads daily CHIRPS data

                Keyword arguments:
                Dir -- 'C:/file/to/path/'
                Startdate -- 'yyyy-mm-dd'
                Enddate -- 'yyyy-mm-dd'
                latlim -- [ymin, ymax] (values must be between -50 and 50)
                lonlim -- [xmin, xmax] (values must be between -180 and 180)
                cores -- The number of cores used to run the routine.
                         It can be 'False' to avoid using parallel computing
                         routines.
                Waitbar -- 1 (Default) will print a waitbar
                """
                print('\nDownload daily CHIRPS rainfall data for the period %s till %s' %(Startdate, Enddate))

                # Download data
                DownloadData(Dir, Startdate, Enddate, latlim, lonlim, Waitbar, cores,
                             TimeCase='daily')

if __name__ == '__main__':
    main(sys.argv)



# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Products/ETref
"""

# import general python modules
import os
import numpy as np

# import watertools modules
from watertools.General import data_conversions as DC
from watertools.General import raster_conversions as RC
from watertools.Products.ETref.SlopeInfluence_ETref import SlopeInfluence

def process_GLDAS(Tmax, Tmin, humidity, surface_pressure):
    """
    This function calculates the actual and saturated vapour pressure by using GLDAS data

    Keyword arguments:
    Tmax -- [] numpy array with the maximal temperature
    Tmin -- [] numpy array with the minimal temperature
    humidity -- [] numpy array with the humidity
    surface_pressure -- [] numpy array with the surface pressure
    """
	# calculate the average temparature based on FAO
    T_mean = (Tmax + Tmin) / 2

	# calculate the slope of saturation
    delta = 4098 * (0.6108  * np.exp(17.27 * T_mean / (T_mean + 237.3))) / (T_mean + 237.3)**2

    # calculate the saturation vapour pressure by using the max and min temparature
    e0_max = 0.6108 * np.exp(17.27 * Tmax / (Tmax + 237.3))
    e0_min = 0.6108 * np.exp(17.27 * Tmin / (Tmin + 237.3))

    # calculate the saturated vapour pressure
    es = (e0_max + e0_min) / 2

    # calculate the max and min relative humidity
    RH_max = np.minimum((1/0.622) * humidity * surface_pressure / e0_min, 1)*100
    RH_min = np.minimum((1/0.622) * humidity * surface_pressure / e0_max, 1)*100

    # calculate the actual vapour pressure
    ea = (e0_min * RH_max/100 + e0_max * RH_min/100) / 2

    return ea, es, delta

def lapse_rate(Dir,temperature_map, DEMmap):
    """
    This function downscales the GLDAS temperature map by using the DEM map

    Keyword arguments:
    temperature_map -- 'C:/' path to the temperature map
    DEMmap -- 'C:/' path to the DEM map
    """

    # calculate average altitudes corresponding to T resolution
    dest = RC.reproject_dataset_example(DEMmap, temperature_map,method = 4)
    DEM_ave_out_name = os.path.join(Dir,'HydroSHED', 'DEM','DEM_ave.tif')
    geo_out, proj, size_X, size_Y = RC.Open_array_info(temperature_map)
    DEM_ave_data = dest.GetRasterBand(1).ReadAsArray()
    DC.Save_as_tiff(DEM_ave_out_name, DEM_ave_data, geo_out, proj)
    dest = None

    # determine lapse-rate [degress Celcius per meter]
    lapse_rate_number = 0.0065

    # open maps as numpy arrays
    dest = RC.reproject_dataset_example(DEM_ave_out_name, DEMmap, method = 2)
    dem_avg=dest.GetRasterBand(1).ReadAsArray()
    dem_avg[dem_avg<0]=0
    dest = None

    # Open the temperature dataset
    dest = RC.reproject_dataset_example(temperature_map, DEMmap, method = 2)
    T=dest.GetRasterBand(1).ReadAsArray()
    dest = None

    # Open Demmap
    demmap = RC.Open_tiff_array(DEMmap)
    dem_avg[demmap<=0]=0
    demmap[demmap==-32768]=np.nan

    # calculate first part
    T = T + ((dem_avg-demmap) * lapse_rate_number)

    return T

def adjust_P(Dir, pressure_map, DEMmap):
    """
    This function downscales the GLDAS air pressure map by using the DEM map

    Keyword arguments:
    pressure_map -- 'C:/' path to the pressure map
    DEMmap -- 'C:/' path to the DEM map
    """

    # calculate average latitudes
    destDEMave = RC.reproject_dataset_example(DEMmap, pressure_map, method = 4)
    DEM_ave_out_name = os.path.join(Dir, 'HydroSHED', 'DEM','DEM_ave.tif')
    geo_out, proj, size_X, size_Y = RC.Open_array_info(pressure_map)
    DEM_ave_data = destDEMave.GetRasterBand(1).ReadAsArray()
    DC.Save_as_tiff(DEM_ave_out_name, DEM_ave_data, geo_out, proj)

    # open maps as numpy arrays
    dest = RC.reproject_dataset_example(DEM_ave_out_name, DEMmap, method = 2)
    dem_avg=dest.GetRasterBand(1).ReadAsArray()
    dest = None

    # open maps as numpy arrays
    dest = RC.reproject_dataset_example(pressure_map, DEMmap, method = 2)
    P=dest.GetRasterBand(1).ReadAsArray()
    dest = None

    demmap = RC.Open_tiff_array(DEMmap)
    dem_avg[demmap<=0]=0
    demmap[demmap==-32768]=np.nan

    # calculate second part
    P = P + (101.3*((293-0.0065*(demmap-dem_avg))/293)**5.26 - 101.3)

    os.remove(DEM_ave_out_name)

    return P

def slope_correct(down_short_hor, pressure, ea, DEMmap, DOY):
    """
    This function downscales the CFSR solar radiation by using the DEM map
    The Slope correction is based on Allen et al. (2006)
    'Analytical integrated functions for daily solar radiation on slope'

    Keyword arguments:
    down_short_hor -- numpy array with the horizontal downwards shortwave radiation
    pressure -- numpy array with the air pressure
    ea -- numpy array with the actual vapour pressure
    DEMmap -- 'C:/' path to the DEM map
    DOY -- day of the year
    """

    # Get Geo Info
    GeoT, Projection, xsize, ysize = RC.Open_array_info(DEMmap)

    minx = GeoT[0]
    miny = GeoT[3] + xsize*GeoT[4] + ysize*GeoT[5]

    x = np.flipud(np.arange(xsize)*GeoT[1] + minx + GeoT[1]/2)
    y = np.flipud(np.arange(ysize)*-GeoT[5] + miny + -GeoT[5]/2)

    # Calculate Extraterrestrial Solar Radiation [W m-2]
    demmap = RC.Open_tiff_array(DEMmap)
    demmap[demmap<0]=0

	# apply the slope correction
    Ra_hor, Ra_slp, sinb, sinb_hor, fi, slope, ID = SlopeInfluence(demmap,y,x,DOY)

    # Calculate atmospheric transmissivity
    Rs_hor = down_short_hor

    # EQ 39
    tau = Rs_hor/Ra_hor

    #EQ 41
    KB_hor = np.zeros(tau.shape) * np.nan

    indice = np.where(tau.flat >= 0.42)
    KB_hor.flat[indice] = 1.56*tau.flat[indice] -0.55

    indice = np.logical_and(tau.flat > 0.175, tau.flat < 0.42)
    KB_hor.flat[indice] = 0.022 - 0.280*tau.flat[indice] + 0.828*tau.flat[indice]**2 + 0.765*tau.flat[indice]**3

    indice = np.where(tau.flat <= 0.175)
    KB_hor.flat[indice] = 0.016*tau.flat[indice]

    # EQ 42
    KD_hor = tau - KB_hor

    Kt=0.7

    #EQ 18
    W = 0.14*ea*pressure + 2.1

    KB0 = 0.98*np.exp((-0.00146*pressure/Kt/sinb)-0.075*(W/sinb)**0.4)
    KB0_hor = 0.98*np.exp((-0.00146*pressure/Kt/sinb_hor)-0.075*(W/sinb_hor)**0.4)

    #EQ 34
    fB = KB0/KB0_hor * Ra_slp/Ra_hor
    fia = (1-KB_hor) * (1 + (KB_hor/(KB_hor+KD_hor))**0.5 * np.sin(slope/2)**3)*fi + fB*KB_hor

    Rs = Rs_hor*(fB*(KB_hor/tau) + fia*(KD_hor/tau) + 0.23*(1-fi))

    Rs[np.isnan(Rs)] = Rs_hor[np.isnan(Rs)]

    Rs_equiv = Rs / np.cos(slope)

    bias = np.nansum(Rs_hor)/np.nansum(Rs_equiv)

    return Rs_equiv, tau, bias



# -*- coding: utf-8 -*-
import sys
from watertools.Collect.MERRA.DataAccess import DownloadData


def main(Dir, Vars, Startdate, Enddate, latlim, lonlim, Waitbar = 1, data_type = ["mean"]):
    """
    This function downloads MERRA daily data for a given variable, time
    interval, and spatial extent.

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    Vars -- ['t2m', 'v2m'] 
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    latlim -- [ymin, ymax]
    lonlim -- [xmin, xmax]
    Waitbar -- 1 (Default) Will print a waitbar
    """
    for Var in Vars:

        if Waitbar == 1:
            print('\nDownloading daily MERRA %s data for the period %s till %s' %(Var, Startdate, Enddate))

        # Download data
        DownloadData(Dir, Var, Startdate, Enddate, latlim, lonlim, "daily_MERRA2", '', Waitbar, data_type)

if __name__ == '__main__':
    main(sys.argv)



# -*- coding: utf-8 -*-
"""
Authors: Tim Hessels
Module: Collect/MYD11
"""

import sys
from watertools.Collect.MYD11.DataAccess import DownloadData

def main(Dir, Startdate, Enddate, latlim, lonlim, Waitbar = 1, cores=False, hdf_library = None, remove_hdf = 1, angle_info = 0, time_info = 0):
    """
    This function downloads MYD11 daily data for the specified time
    interval, and spatial extent.

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    latlim -- [ymin, ymax]
    lonlim -- [xmin, xmax]
    Waitbar -- 1 (Default) will print a waitbar
    hdf_library -- string, if all the hdf files are already stored on computer
                    define directory to the data here
    remove_hdf -- 1 (Default), if 1 remove all the downloaded hdf files in the end
    """

    print('\nDownload daily MODIS land surface temperature data for period %s till %s' %(Startdate, Enddate))
    TimeStep = 1
    DownloadData(Dir, Startdate, Enddate, latlim, lonlim, TimeStep, Waitbar, cores, hdf_library, remove_hdf, angle_info, time_info)

if __name__ == '__main__':
    main(sys.argv)


# -*- coding: utf-8 -*-
'''
Authors: Tim Hessels
Module: Products/SoilGrids
'''

# import general python modules
import os
import numpy as np

# import watertools modules
from watertools.General import data_conversions as DC
from watertools.General import raster_conversions as RC

def Topsoil(Dir, latlim, lonlim):
    """
    This function calculates the n van genuchten soil characteristic (15cm)

    Keyword arguments:
    Dir -- 'C:/' path to the map
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    """
	
    print('/nCreate n van genuchten map of the topsoil from SoilGrids')
    
    # Define parameters to define the topsoil
    SL = "sl3"
	
    Calc_Property(Dir, latlim, lonlim, SL)
	
    return

def Subsoil(Dir, latlim, lonlim):
    """
    This function calculates the subsoil n van genuchten soil characteristic (100cm)

    Keyword arguments:
    Dir -- 'C:/' path to the WA map
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    """
	
    print('/nCreate n van genuchten map of the subsoil from SoilGrids')
    
	 # Define parameters to define the subsoil	
    SL = "sl6"
	
    Calc_Property(Dir, latlim, lonlim, SL)
	
    return
	
def Calc_Property(Dir, latlim, lonlim, SL):	
    
    import watertools
    
    # Define level
    if SL == "sl3":
        level = "Topsoil"
    elif SL == "sl6":
        level = "Subsoil" 
    
    # check if you need to download
    filename_out_thetasat = os.path.join(Dir, 'SoilGrids', 'Theta_Sat' ,'Theta_Sat2_%s_SoilGrids_kg-kg.tif' %level)
    if not os.path.exists(filename_out_thetasat):
       if SL == "sl3":
           watertools.Products.SoilGrids.Theta_Sat2.Topsoil(Dir, latlim, lonlim)
       elif SL == "sl6":
           watertools.Products.SoilGrids.Theta_Sat2.Subsoil(Dir, latlim, lonlim)

    filedir_out_n_genuchten = os.path.join(Dir, 'SoilGrids', 'N_van_genuchten')
    if not os.path.exists(filedir_out_n_genuchten):
        os.makedirs(filedir_out_n_genuchten)   
             
    # Define n van genuchten output
    filename_out_ngenuchten = os.path.join(filedir_out_n_genuchten ,'N_genuchten_%s_SoilGrids_-.tif' %level)

    if not os.path.exists(filename_out_ngenuchten):
            
        # Get info layer
        geo_out, proj, size_X, size_Y = RC.Open_array_info(filename_out_thetasat)
        
        # Open dataset
        theta_sat = RC.Open_tiff_array(filename_out_thetasat)
        
        # Calculate n van genuchten
        n_van_genuchten = np.ones(theta_sat.shape) * -9999   
        n_van_genuchten = 166.63*theta_sat**4-387.72*theta_sat**3+340.55*theta_sat**2-133.07*theta_sat+20.739
		
        # Save as tiff
        DC.Save_as_tiff(filename_out_ngenuchten, n_van_genuchten, geo_out, proj)
    return           
           
           
	
	





# -*- coding: utf-8 -*-
'''
Authors: Tim Hessels
Module: Products/ETref
'''
# import general python modules
import os
import pandas as pd
from joblib import Parallel, delayed

# import watertools modules
from watertools.General import raster_conversions as RC
from watertools.General import data_conversions as DC
from watertools.Products.ETref.CalcETref import calc_ETref


def SetVariables(Dir, Startdate, Enddate, latlim, lonlim, pixel_size, cores, LANDSAF, Waitbar):
    """
    This function starts to calculate ETref (daily) data based on Hydroshed, GLDAS, and (CFSR/LANDSAF) in parallel or single core

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    Startdate -- 'yyyy-mm-dd'
    Enddate -- 'yyyy-mm-dd'
    latlim -- [ymin, ymax] (values must be between -60 and 60)
    lonlim -- [xmin, xmax] (values must be between -180 and 180)
    pixel_size -- The output pixel size
    cores -- The number of cores used to run the routine.
             It can be 'False' to avoid using parallel computing
			routines.
    LANDSAF -- if LANDSAF data must be used it is 1
    SourceLANDSAF -- the path to the LANDSAF files
    Waitbar -- 1 (Default) will print the waitbar
    """
    # Make an array of the days of which the ET is taken
    Dates = pd.date_range(Startdate,Enddate,freq = 'D')

    # Create Waitbar
    if Waitbar == 1:
        import watertools.Functions.Random.WaitbarConsole as Waitbar
        total_amount = len(Dates)
        amount = 0
        Waitbar.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)

    # Pass variables to parallel function and run
    args = [Dir, lonlim, latlim, pixel_size, LANDSAF]
    if not cores:
        for Date in Dates:
            ETref(Date, args)
            if Waitbar == 1:
                amount += 1
                Waitbar.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)
        results = True
    else:
        results = Parallel(n_jobs=cores)(delayed(ETref)(Date, args)
                                         for Date in Dates)
    return results

def ETref(Date, args):
    """
    This function starts to calculate ETref (daily) data based on Hydroshed, GLDAS, and (CFSR/LANDSAF) in parallel or single core

    Keyword arguments:
    Date -- panda timestamp
    args -- includes all the parameters that are needed for the ETref
	"""

	# unpack the arguments
    [Dir, lonlim, latlim, pixel_size, LANDSAF] = args

    # Set the paths
    nameTmin='Tair-min_GLDAS-NOAH_C_daily_' + Date.strftime('%Y.%m.%d') + ".tif"
    tmin_str=os.path.join(Dir,'Weather_Data','Model','GLDAS','daily','tair_f_inst','min',nameTmin )

    nameTmax='Tair-max_GLDAS-NOAH_C_daily_' + Date.strftime('%Y.%m.%d') + ".tif"
    tmax_str=os.path.join(Dir,'Weather_Data','Model','GLDAS','daily','tair_f_inst','max',nameTmax )

    nameHumid='Hum_GLDAS-NOAH_kg-kg_daily_'+ Date.strftime('%Y.%m.%d') + ".tif"
    humid_str=os.path.join(Dir,'Weather_Data','Model','GLDAS','daily','qair_f_inst','mean',nameHumid )

    namePress='P_GLDAS-NOAH_kpa_daily_'+ Date.strftime('%Y.%m.%d') + ".tif"
    press_str=os.path.join(Dir,'Weather_Data','Model','GLDAS','daily','psurf_f_inst','mean',namePress )

    nameWind='W_GLDAS-NOAH_m-s-1_daily_'+ Date.strftime('%Y.%m.%d') + ".tif"
    wind_str=os.path.join(Dir,'Weather_Data','Model','GLDAS','daily','wind_f_inst','mean',nameWind )

    if LANDSAF==1:

        nameShortClearname = 'ShortWave_Clear_Daily_W-m2_' + Date.strftime('%Y-%m-%d') + '.tif'
        input2_str=os.path.join(Dir,'Landsaf_Clipped','Shortwave_Clear_Sky',nameShortClearname)

        nameShortNetname = 'ShortWave_Net_Daily_W-m2_' + Date.strftime('%Y-%m-%d') + '.tif'
        input1_str=os.path.join(Dir,'Landsaf_Clipped','Shortwave_Net',nameShortNetname)

        input3_str='not'

    else:
        if Date<pd.Timestamp(pd.datetime(2011, 4, 1)):

            nameDownLong='DLWR_CFSR_W-m2_' + Date.strftime('%Y.%m.%d') + ".tif"
            input2_str=os.path.join(Dir,'Radiation','CFSR',nameDownLong)

            nameDownShort='DSWR_CFSR_W-m2_' + Date.strftime('%Y.%m.%d') + ".tif"
            input1_str=os.path.join(Dir,'Radiation','CFSR',nameDownShort)

            nameUpLong='ULWR_CFSR_W-m2_' + Date.strftime('%Y.%m.%d') + ".tif"
            input3_str=os.path.join(Dir,'Radiation','CFSR',nameUpLong)

        else:
            nameDownLong='DLWR_CFSRv2_W-m2_' + Date.strftime('%Y.%m.%d') + ".tif"
            input2_str=os.path.join(Dir,'Radiation','CFSRv2',nameDownLong)

            nameDownShort='DSWR_CFSRv2_W-m2_' + Date.strftime('%Y.%m.%d') + ".tif"
            input1_str=os.path.join(Dir,'Radiation','CFSRv2',nameDownShort)

            nameUpLong='ULWR_CFSRv2_W-m2_' + Date.strftime('%Y.%m.%d') + ".tif"
            input3_str=os.path.join(Dir,'Radiation','CFSRv2',nameUpLong)

   # The day of year
    DOY=Date.dayofyear

    # Load DEM
    if not pixel_size:
        DEMmap_str=os.path.join(Dir,'HydroSHED','DEM','DEM_HydroShed_m_3s.tif')
    else:
        DEMmap_str=os.path.join(Dir,'HydroSHED','DEM','DEM_HydroShed_m_3s.tif')
        dest, ulx, lry, lrx, uly, epsg_to = RC.reproject_dataset_epsg(DEMmap_str, pixel_spacing = pixel_size, epsg_to=4326, method = 2)
        DEMmap_str=os.path.join(Dir,'HydroSHED','DEM','DEM_HydroShed_m_reshaped_for_ETref.tif')
        DEM_data = dest.GetRasterBand(1).ReadAsArray()
        geo_dem = [ulx, pixel_size, 0.0, uly, 0.0, - pixel_size]
        DC.Save_as_tiff(name=DEMmap_str, data=DEM_data, geo=geo_dem, projection='4326')

    # Calc ETref
    ETref = calc_ETref(Dir, tmin_str, tmax_str, humid_str, press_str, wind_str, input1_str, input2_str, input3_str, DEMmap_str, DOY)

    # Make directory for the MODIS ET data
    output_folder=os.path.join(Dir,'ETref','Daily')
    if not os.path.exists(output_folder):
        os.makedirs(output_folder)

    # Create the output names
    NameETref='ETref_mm-day-1_daily_'+Date.strftime('%Y.%m.%d') + '.tif'
    NameEnd=os.path.join(output_folder,NameETref)

    # Collect geotiff information
    geo_out, proj, size_X, size_Y = RC.Open_array_info(DEMmap_str)

    # Create daily ETref tiff files
    DC.Save_as_tiff(name=NameEnd, data=ETref, geo=geo_out, projection=proj)







# -*- coding: utf-8 -*-
"""
Created on Thu Jul 23 13:24:54 2020

@author: timhe
"""

# General Python modules
import numpy as np
import os
import glob
import pandas as pd
from osgeo import gdal

def Nearest_Interpolate(Dir_in, Startdate, Enddate, Dir_out=None):
    """
    This functions calculates monthly tiff files based on the daily tiff files.
    (will calculate the total sum)

    Parameters
    ----------
    Dir_in : str
        Path to the input data
    Startdate : str
        Contains the start date of the model 'yyyy-mm-dd'
    Enddate : str
        Contains the end date of the model 'yyyy-mm-dd'
    Dir_out : str
        Path to the output data, default is same as Dir_in

    """
    # import WA+ modules
    import watertools.General.data_conversions as DC
    import watertools.General.raster_conversions as RC

    # Change working directory
    os.chdir(Dir_in)

    # Define end and start date
    Dates = pd.date_range(Startdate, Enddate, freq='D')

    # Find all hourly files
    files = glob.glob('*hourly*.tif')

    # Get array information and define projection
    geo_out, proj, size_X, size_Y = RC.Open_array_info(files[0])
    if int(proj.split('"')[-2]) == 4326:
        proj = "WGS84"

    # Get the No Data Value
    dest = gdal.Open(files[0])
    NDV = dest.GetRasterBand(1).GetNoDataValue()

    # Define output directory
    if Dir_out is None:
	     Dir_out = Dir_in

    if not os.path.exists(Dir_out):
	     os.makedirs(Dir_out)

    # loop over the months and sum the days
    for date in Dates:
        Year = date.year
        Month = date.month
        Day = date.day
        files_one_day = glob.glob('*hourly*%d.%02d.%02d*.tif' % (Year, Month, Day))

        # Create empty arrays
        Day_data = np.ones([24, size_Y, size_X]) * np.nan

        if len(files_one_day) != 24:
            print("One day is missing!!! month %s year %s" %(Month, Year))
        
        i = 0
        for file_one_day in files_one_day:
            file_path = os.path.join(Dir_in, file_one_day)

            Hour_data = RC.Open_tiff_array(file_path)
            Hour_data[Hour_data == -9999] = np.nan
            Day_data[i, :, :] = Hour_data[None, :, :]
            i+=1
            
        Day_data = np.nanmean(Day_data, axis = 0)

        # Define output name
        output_name = os.path.join(Dir_out, file_one_day
                                   .replace('hourly', 'daily')
                                   .replace('hour', 'day'))

        output_name = output_name[:-19] + '%d.%02d.%02d.tif' % (date.year, date.month, date.day)

        # Save tiff file
        DC.Save_as_tiff(output_name, Day_data, geo_out, proj)

    return



# -*- coding: utf-8 -*-
import sys
import os
from watertools.Collect.ESACCI.DataAccess import DownloadData

def main(Dir, latlim, lonlim, Waitbar = 1):
    """
    This function downloads ESACCI daily data for a given variable, time
    interval, and spatial extent.

    Keyword arguments:
    Dir -- 'C:/file/to/path/'
    latlim -- [ymin, ymax]
    lonlim -- [xmin, xmax]
    Waitbar -- 1 (Default) Will print a waitbar
    """
	
    # Create directory if not exists for the output
    output_folder = os.path.join(Dir, 'ESACCI', 'LU')
    if not os.path.exists(output_folder):
        os.makedirs(output_folder)

    # Define the output map and create this if not exists
    nameEnd = os.path.join(output_folder, 'LU_ESACCI.tif')

    if not os.path.exists(nameEnd):

        # Create Waitbar
        if Waitbar == 1:
            print('\nDownload ESACCI landuse map')
            import watertools.Functions.Random.WaitbarConsole as WaitbarConsole
            total_amount = 1
            amount = 0
            WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)

        # Download and process the data
        DownloadData(output_folder, latlim, lonlim, Waitbar)

        if Waitbar == 1:
            amount = 1
            WaitbarConsole.printWaitBar(amount, total_amount, prefix = 'Progress:', suffix = 'Complete', length = 50)

    else:
        if Waitbar == 1:
            print("\nESACCI LU map already exists in output folder")

if __name__ == '__main__':
    main(sys.argv)

